@@ -1,6 +1,6 @@
 ## Domain Registry Interface, Main entry point
 ##
-## Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005-2011 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,19 +18,21 @@
 package Net::DRI;
 
 use strict;
+use warnings;
 
 require UNIVERSAL::require;
 
 use Net::DRI::Cache;
 use Net::DRI::Registry;
 use Net::DRI::Util;
+use Net::DRI::Exception;
 
 use base qw(Class::Accessor::Chained::Fast Net::DRI::BaseClass);
 __PACKAGE__->mk_ro_accessors(qw/trid_factory logging cache/);
 
 our $AUTOLOAD;
-our $VERSION='0.92_01';
-our $CVS_REVISION=do { my @r=(q$Revision: 1.34 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION='0.96_01';
+our $CVS_REVISION=do { my @r=(q$Revision: 1.39 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 our $RUNNING_POE=(exists($INC{'POE.pm'}))? $POE::Kernel::poe_kernel : undef;
 
 =pod
@@ -41,13 +43,17 @@ Net::DRI - Interface to Domain Name Registries/Registrars/Resellers
 
 =head1 VERSION
 
-This documentation refers to Net::DRI version 0.92_01 (DEVELOPMENT RELEASE)
+This documentation refers to Net::DRI version 0.96_01
 
 =head1 SYNOPSIS
 
 	use Net::DRI;
 	my $dri=Net::DRI->new({ cache_ttl => 10, trid_factory => ..., logging => .... });
 
+	... various operations ...
+
+	$dri->end();
+
 =head1 DESCRIPTION
 
 Net::DRI is a Perl library to access services offered by domain name
@@ -58,10 +64,10 @@ an abstraction over multiple providers, with multiple policies, transports
 and protocols all used through a uniform API.
 
 It is an object-oriented framework implementing RRP (RFC 2832/3632),
-EPP (core EPP in RFC 4930/4931/4932/4933/4934, extensions in
+EPP (core EPP in RFC 5730/5731/5732/5733/5734 aka STD69, extensions in
 RFC 3915/4114/4310/5076 and various extensions of ccTLDs/gTLDs
-- currently more than 30 TLDs are directly supported with extensions),
-RRI (.DE registration protocol), Whois, DAS (Domain Availability Service used by .BE, .EU),
+- currently more than 60 TLDs are directly supported with extensions),
+RRI (.DE registration protocol), Whois, DAS (Domain Availability Service used by .BE, .EU, .AU, .NL),
 IRIS (RFC3981) DCHK (RFC5144) over LWZ (RFC4993) for .DE currently and XCP (RFC4992),
 .FR/.RE email and webservices interface, and resellers interface of some registrars
 (Gandi, OpenSRS, etc.).
@@ -106,10 +112,14 @@ it will not apply to already existing objects (registry profiles and transports)
 =head2 logging()
 
 This is an accessor to the underlying Logging object. During the C<new()> call you can
-provide the object, or just a string ("null", "stderr", or "files"), or a reference to an array
+provide the object, or just a string ("null", "stderr", "files" or "syslog" which are the
+current logging modules available in Net::DRI), or a reference to an array
 with the first parameter a string (same as previously) and the second parameter a reference to
 an hash with data needed by the logging class used (see for example L<Net::DRI::Logging::Files>).
 
+If you want to log the application data (what is exchanged with remote server, such as EPP XML streams),
+you need to use logging level of 'notice', or higher.
+
 =head2 cache()
 
 This is an accessor to the underlying Cache object. See L<Net::DRI::Cache>.
@@ -135,7 +145,7 @@ and various contributors (see Changes file and web page above)
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005-2011 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 =head1 LICENSE
@@ -169,24 +179,26 @@ sub new
  my ($logname,@logdata);
  if (exists $rh->{logging})
  {
-  if (ref $rh->{logging} eq 'ARRAY')
-  {
-   ($logname,@logdata)=@{$rh->{logging}};
-  } else
-  {
-   $logname=$rh->{logging};
-  }
+  ($logname,@logdata)=ref $rh->{logging} eq 'ARRAY' ? @{$rh->{logging}} : ($rh->{logging});
  } else
  {
   $logname='null';
  }
- if ($logname !~ m/::/) { $logname='Net::DRI::Logging::'.ucfirst($logname); }
- $logname->require or Net::DRI::Exception::err_failed_load_module('DRI',$logname,$@);
+ if ($logname !~ m/^\+/) { $logname='Net::DRI::Logging::'.ucfirst($logname); }
+ $logname->require() or Net::DRI::Exception::err_failed_load_module('DRI',$logname,$@);
  $self->{logging}=$logname->new(@logdata);
 
  bless($self,$class);
  $self->logging()->setup_channel(__PACKAGE__,'core');
+ $self->log_output('notice','core','Successfully created Net::DRI object with logging='.$logname);
+ return $self;
+}
 
+sub add_current_registry
+{
+ my ($self,@p)=@_;
+ $self->add_registry(@p);
+ $self->target($self->{last_registry});
  return $self;
 }
 
@@ -194,45 +206,47 @@ sub add_registry
 {
  my ($self,$reg,@data)=@_;
  Net::DRI::Exception::usererr_insufficient_parameters('add_registry needs a registry name') unless Net::DRI::Util::all_valid($reg);
- $reg='Net::DRI::DRD::'.$reg unless ($reg=~m/::/);
- $reg->require or Net::DRI::Exception::err_failed_load_module('DRI',$reg,$@);
+ $reg='Net::DRI::DRD::'.$reg unless $reg=~m/^\+/;
+ $reg->require() or Net::DRI::Exception::err_failed_load_module('DRI',$reg,$@);
 
  my $drd=$reg->new(@data);
- Net::DRI::Exception->die(1,'DRI',9,'Failed to initialize registry '.$reg) unless ($drd && ref($drd));
+ Net::DRI::Exception->die(1,'DRI',9,'Failed to initialize registry '.$reg) unless ($drd && ref $drd);
 
- Net::DRI::Exception::err_method_not_implemented('name() in '.$reg) unless $drd->can('name');
+ Net::DRI::Exception::method_not_implemented('name',$reg) unless $drd->can('name');
  my $regname=$drd->name();
- Net::DRI::Exception->die(1,'DRI',10,'No dot allowed in registry name: '.$regname) unless (index($regname,'.')==-1);
- Net::DRI::Exception->die(1,'DRI',11,'New registry name already in use') if (exists($self->{registries}->{$regname}));
+ Net::DRI::Exception->die(1,'DRI',10,'No dot allowed in registry name: '.$regname) unless index($regname,'.') == -1;
+ Net::DRI::Exception->die(1,'DRI',11,'New registry name already in use') if exists $self->{registries}->{$regname};
 
  my $ndr=Net::DRI::Registry->new($regname,$drd,$self->{cache},$self->{trid_factory},$self->{logging});
- $self->{registries}->{$regname}=$ndr;
-
- Net::DRI::Exception::err_method_not_implemented('tlds() in '.$reg) unless $drd->can('tlds');
+ Net::DRI::Exception::method_not_implemented('tlds',$reg) unless $drd->can('tlds');
  foreach my $tld ($drd->tlds())
  {
   $tld=lc($tld);
-  $self->{tlds}->{$tld}=[] unless exists($self->{tlds}->{$tld});
-  push @{$self->{tlds}->{$tld}},$regname
+  $self->{tlds}->{$tld}=[] unless exists $self->{tlds}->{$tld};
+  push @{$self->{tlds}->{$tld}},$regname;
  }
 
+ $self->log_output('notice','core','Successfully added registry "'.$regname.'"');
+ $self->{registries}->{$regname}=$ndr;
+ $self->{last_registry}=$regname;
  return $self;
 }
 
 sub del_registry
 {
  my ($self,$name)=@_;
- if (defined($name))
+ if (defined $name)
  {
-  err_registry_name_does_not_exist($name) unless (exists($self->{registries}->{$name}));
+  err_registry_name_does_not_exist($name) unless exists $self->{registries}->{$name};
  } else
  {
-  err_no_current_registry() unless (defined($self->{current_registry}));
+  err_no_current_registry() unless defined $self->{current_registry};
   $name=$self->{current_registry};
  }
  $self->{registries}->{$name}->end();
  delete($self->{registries}->{$name});
- $self->{current_registry}=undef if ($self->{current_registry} eq $name);
+ $self->{current_registry}=undef if $self->{current_registry} eq $name;
+ $self->log_output('notice','core','Successfully deleted registry "'.$name.'"');
  return $self;
 }
 
@@ -264,7 +278,7 @@ sub registry
  err_no_current_registry()                  unless (defined($regname) && $regname);
  err_registry_name_does_not_exist($regname) unless (exists($self->{registries}->{$regname}));
  my $ndr=$self->{registries}->{$regname};
- return wantarray()? ($regname,$ndr) : $ndr;
+ return wantarray? ($regname,$ndr) : $ndr;
 }
 
 sub tld2reg
@@ -278,23 +292,29 @@ sub tld2reg
  return @t;
 }
 
+sub installed_registries
+{
+ return qw/AdamsNames AERO AFNIC AG ARNES ASIA AT AU BE BIZ BookMyName BR BZ CAT CentralNic CIRA CoCCA COOP CZ DENIC EURid Gandi GL HN IENUMAT IM INFO IRegistry ISPAPI IT LC LU ME MN MOBI NAME Nominet NO NU OpenSRS ORG OVH PL PRO PT SC SE SIDN SWITCH TRAVEL US VC VNDS WS/;
+}
+
 ####################################################################################################
+
 sub target
 {
  my ($self,$driver,$profile)=@_;
 
  ## Try to convert if given a domain name or a tld instead of a driver's name
- if (defined($driver) && !exists($self->{registries}->{$driver}))
+ if (defined $driver && ! exists $self->{registries}->{$driver})
  {
   my @t=$self->tld2reg($driver);
   Net::DRI::Exception->die(0,'DRI',7,'Registry not found for domain name/TLD '.$driver) unless (@t==1);
   $driver=$t[0];
  }
 
- $driver=$self->registry_name() unless defined($driver);
- err_registry_name_does_not_exist($driver) unless defined($driver) && $driver;
+ $driver=$self->registry_name() unless defined $driver;
+ err_registry_name_does_not_exist($driver) unless defined $driver && $driver;
 
- if (defined($profile))
+ if (defined $profile)
  {
   $self->{registries}->{$driver}->target($profile);
  }
@@ -313,8 +333,8 @@ sub AUTOLOAD
  $attr=~s/.*:://;
  return unless $attr=~m/[^A-Z]/; ## skip DESTROY and all-cap methods
 
- my $ndr=$self->registry(); ## This is a Net::DRI::Registry object
- Net::DRI::Exception::err_method_not_implemented($attr.' in '.$ndr) unless (ref($ndr) && $ndr->can($attr));
+ my ($name,$ndr)=$self->registry();
+ Net::DRI::Exception::method_not_implemented($attr,$ndr) unless ref $ndr && $ndr->can($attr);
  $self->log_output('debug','core','Calling '.$attr.' from Net::DRI');
  return $ndr->$attr(@_); ## is goto beter here ?
 }
@@ -322,18 +342,25 @@ sub AUTOLOAD
 sub end
 {
  my $self=shift;
-
- foreach my $v (values(%{$self->{registries}}))
+ while(my ($name,$v)=each(%{$self->{registries}}))
  {
   $v->end() if (ref($v) && $v->can('end'));
+  $self->log_output('notice','core','Successfully ended registry "'.$name.'"');
   $v={};
  }
  $self->{tlds}={};
  $self->{registries}={};
  $self->{current_registry}=undef;
+ if (defined $self->{logging})
+ {
+  $self->log_output('notice','core','Successfully ended Net::DRI object');
+  $self->{logging}=undef;
+ }
+
+ return 1; ## this makes it easy to test if everything before was ok or not, if we are inside an eval {} and $dri->end() is the last operation inside the eval block
 }
 
-sub DESTROY { shift->end(); }
+sub DESTROY { my $self=shift; $self->end(); }
 
 ####################################################################################################
 
@@ -344,20 +371,24 @@ use base qw/Net::DRI/;
 our $AUTOLOAD;
 
 ## Some methods may die in Net::DRI, we specifically trap them
-sub add_registry { my $r; eval { $r=shift->SUPER::add_registry(@_); }; return $r unless $@; die(ref($@)? $@->as_string() : $@); }
-sub del_registry { my $r; eval { $r=shift->SUPER::del_registry(@_); }; return $r unless $@; die(ref($@)? $@->as_string() : $@); }
-sub registry { my $r; eval { $r=shift->SUPER::registry(@_); }; return $r unless $@; die(ref($@)? $@->as_string() : $@); }
-sub target { my $r; eval { $r=shift->SUPER::target(@_); }; return $r unless $@; die(ref($@)? $@->as_string() : $@); }
-sub end { my $r; eval { $r=shift->SUPER::end(@_); }; return $r unless $@; die(ref($@)? $@->as_string() : $@); }
+sub add_registry { my ($self,@p)=@_; my $r; my $ok=eval { $r=$self->SUPER::add_registry(@p); 1; }; return $r if $ok; die(ref $@ ? $@->as_string() : $@); }
+sub del_registry { my ($self,@p)=@_; my $r; my $ok=eval { $r=$self->SUPER::del_registry(@p); 1; }; return $r if $ok; die(ref $@ ? $@->as_string() : $@); }
+sub registry     { my ($self,@p)=@_; my @r; my $ok=eval { @r=$self->SUPER::registry(@p); 1; }; if ($ok) { return wantarray ? @r : $r[0]; } die(ref $@ ? $@->as_string() : $@); }
+sub target       { my ($self,@p)=@_; my $r; my $ok=eval { $r=$self->SUPER::target(@p); 1; }; return $r if $ok; die(ref $@ ? $@->as_string() : $@); }
+sub end          { my ($self,@p)=@_; my $r; my $ok=eval { $r=$self->SUPER::end(@p); 1; }; return $r if $ok; die(ref $@ ? $@->as_string() : $@); }
 
 sub AUTOLOAD
 {
  my $self=shift;
- my $r;
+ my @r;
  $Net::DRI::AUTOLOAD=$AUTOLOAD;
- eval { $r=$self->SUPER::AUTOLOAD(@_); };
- return $r unless $@;
- die(ref($@)? $@->as_string() : $@);
+ my $ok=eval { @r=$self->SUPER::AUTOLOAD(@_); 1; };
+ if (! $ok)
+ {
+  my $err=$@;
+  die(ref $err ? $err->as_string() : $err);
+ }
+ return wantarray ? @r : $r[0];
 }
 
 ####################################################################################################
