@@ -1,7 +1,7 @@
 ## Domain Registry Interface, .AT policy
 ## Contributed by Michael Braunoeder from NIC.AT <mib@nic.at>
 ##
-## Copyright (c) 2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2006-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -19,11 +19,15 @@
 package Net::DRI::DRD::AT;
 
 use strict;
+use warnings;
+
 use base qw/Net::DRI::DRD/;
 
 use DateTime::Duration;
+use Net::DRI::Data::Contact::AT;
+use Net::DRI::Util;
 
-our $VERSION=do { my @r=(q$Revision: 1.6 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.9 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 __PACKAGE__->make_exception_for_unavailable_operations(qw/domain_transfer_accept domain_transfer_refuse domain_renew contact_transfer_stop contact_transfer_query contact_transfer_accept contact_transfer_refuse contact_check/);
 
@@ -55,7 +59,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2006-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -83,51 +87,39 @@ sub periods  { return map { DateTime::Duration->new(years => $_) } (1); }
 sub name     { return 'NICAT'; }
 sub tlds     { return ('at'); }
 sub object_types { return ('domain','contact'); }
-
-sub transport_protocol_compatible
-{
- my ($self,$to,$po)=@_;
- my $pn=$po->name();
- my $tn=$to->name();
-
- return 1 if (($pn eq 'EPP') && ($tn eq 'socket_inet'));
- return 1 if (($pn eq 'Whois') && ($tn eq 'socket_inet'));
- return;
-}
+sub profile_types { return qw/epp whois/; }
 
 sub transport_protocol_default
 {
- my ($drd,$ndr,$type,$ta,$pa)=@_;
- $type='epp' if (!defined($type) || ref($type));
- return Net::DRI::DRD::_transport_protocol_default_epp('Net::DRI::Protocol::EPP::Extensions::AT',$ta,$pa) if ($type eq 'epp');
- return ('Net::DRI::Transport::Socket',[{%Net::DRI::DRD::PROTOCOL_DEFAULT_WHOIS,remote_host=>'whois.nic.at'}],'Net::DRI::Protocol::Whois',[]) if (lc($type) eq 'whois');
+ my ($self,$type)=@_;
+
+ return ('Net::DRI::Transport::Socket',{},'Net::DRI::Protocol::EPP::Extensions::AT',{})              if $type eq 'epp';
+ return ('Net::DRI::Transport::Socket',{remote_host=>'whois.nic.at'},'Net::DRI::Protocol::Whois',{}) if $type eq 'whois';
+ return;
+}
+
+sub set_factories
+{
+ my ($self,$po)=@_;
+ $po->factories('contact',sub { return Net::DRI::Data::Contact::AT->new(@_); });
 }
 
 ####################################################################################################
-## TODO : this should be converted to the new framework !
+
 sub verify_name_domain
 {
- my ($self,$ndr,$domain)=@_;
- $domain=$ndr unless (defined($ndr) && $ndr && (ref($ndr) eq 'Net::DRI::Registry'));
-
- my @splited = split /\./,$domain;
- my $count = @splited;
- $count--;
- my $r=$self->SUPER::check_name($domain,$count);
- return $r if ($r);
-
- return 10 unless $self->is_my_tld($ndr,$domain,0);
-
- return 0;
+ my ($self,$ndr,$domain,$op)=@_;
+ return $self->_verify_name_rules($domain,$op,{check_name_no_dots => 1, ## is this correct?
+                                               my_tld_not_strict => 1, ## is this correct?
+                                              });
 }
 
 sub domain_withdraw
 {
  my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'withdraw');
-
- $rd={} unless (defined($rd) && (ref($rd) eq 'HASH'));
- $rd->{transactionname} = 'withdraw';
+ $self->enforce_domain_name_constraints($ndr,$domain,'withdraw');
+ $rd=Net::DRI::Util::create_params('domain_withdraw',$rd);
+ $rd->{transactionname}='withdraw';
 
  my $rc=$ndr->process('domain','nocommand',[$domain,$rd]);
  return $rc;
@@ -136,10 +128,9 @@ sub domain_withdraw
 sub domain_transfer_execute
 {
  my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'transfer_execute');
-
- $rd={} unless (defined($rd) && (ref($rd) eq 'HASH'));
- $rd->{transactionname} = 'transfer_execute';
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_execute');
+ $rd=Net::DRI::Util::create_params('domain_transfer_execute',$rd);
+ $rd->{transactionname}='transfer_execute';
 
  my $rc=$ndr->process('domain','nocommand',[$domain,$rd]);
  return $rc;
