@@ -1,6 +1,6 @@
 ## Domain Registry Interface, CentralNic Registry Driver
 ##
-## Copyright (c) 2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2008-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,12 +18,16 @@
 package Net::DRI::DRD::CentralNic;
 
 use strict;
+use warnings;
+
 use base qw/Net::DRI::DRD/;
 
 use DateTime::Duration;
 use DateTime;
 
-our $VERSION=do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+use Net::DRI::Util;
+
+our $VERSION=do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -53,7 +57,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2008-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -79,25 +83,16 @@ sub new
 
 sub periods      { return map { DateTime::Duration->new(years => $_) } (2..10); }
 sub name         { return 'CentralNic'; }
-sub tlds         { return (qw/la uk.net se.net gb.net/,map { $_.'.com' } qw/eu uk us cn de jpn kr no za br ar ru sa se hu gb qc uy ae/); } ## see https://www.centralnic.com/names/domains
+sub tlds         { return (qw/la uk.net se.net gb.net/,map { $_.'.com' } qw/eu uk us cn de jpn kr no za br ar ru sa se hu gb qc uy ae gr/); } ## see https://www.centralnic.com/names/domains
 sub object_types { return ('domain','ns','contact'); }
-
-sub transport_protocol_compatible 
-{
- my ($self,$to,$po)=@_;
- my $pn=$po->name();
- my $pv=$po->version();
- my $tn=$to->name();
-
- return 1 if (($pn eq 'EPP') && ($tn eq 'socket_inet'));
- return;
-}
+sub profile_types { return qw/epp/; }
 
 sub transport_protocol_default
 {
- my ($drd,$ndr,$type,$ta,$pa)=@_;
- $type='epp' if (!defined($type) || ref($type));
- return Net::DRI::DRD::_transport_protocol_default_epp('Net::DRI::Protocol::EPP::Extensions::CentralNic',$ta,$pa) if ($type eq 'epp');
+ my ($self,$type)=@_;
+
+ return ('Net::DRI::Transport::Socket',{},'Net::DRI::Protocol::EPP::Extensions::CentralNic',{}) if $type eq 'epp';
+ return;
 }
 
 ####################################################################################################
@@ -111,29 +106,18 @@ sub verify_name_domain
                                               });
 }
 
-## We can not start a transfer, if domain name has already been transfered less than 15 days ago.
 sub verify_duration_transfer
 {
  my ($self,$ndr,$duration,$domain,$op)=@_;
- ($duration,$domain,$op)=($ndr,$duration,$domain) unless (defined($ndr) && $ndr && (ref($ndr) eq 'Net::DRI::Registry'));
 
- return 0 unless ($op eq 'start'); ## we are not interested by other cases, they are always OK
- my $rc=$self->domain_info($ndr,$domain,{hosts=>'none'});
- return 1 unless ($rc->is_success());
- my $trdate=$ndr->get_info('trDate');
- return 0 unless ($trdate && $trdate->isa('DateTime'));
- 
- my $now=DateTime->now(time_zone => $trdate->time_zone()->name());
- my $cmp=DateTime->compare($now,$trdate+DateTime::Duration->new(days => 15));
- return ($cmp == 1)? 0 : 1; ## we must have : now > transferdate + 15days
- ## we return 0 if OK, anything else if not
+ return $self->_verify_duration_transfer_15days($ndr,$duration,$domain,$op);
 }
 
 sub verify_duration_renew
 {
  my ($self,$ndr,$duration,$domain,$curexp)=@_;
- ($duration,$domain,$curexp)=($ndr,$duration,$domain) unless (defined($ndr) && $ndr && (ref($ndr) eq 'Net::DRI::Registry'));
- return 0 unless (defined($duration) && defined($curexp) && UNIVERSAL::isa($curexp,'DateTime'));
+
+ return 0 unless (defined $duration && defined $curexp && Net::DRI::Util::is_class($curexp,'DateTime'));
  my $newexp=$curexp+$duration; ## New expiration
  my $max=DateTime->new(year => 2037, month => 1, day => 1, time_zone => $curexp->time_zone()->name());
  my $cmp=DateTime->compare($newexp,$max);
@@ -146,7 +130,7 @@ sub verify_duration_renew
 sub domain_release
 {
  my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'release');
+ $self->enforce_domain_name_constraints($ndr,$domain,'release');
 
  return $ndr->process('domain','release',[$domain,$rd]);
 }
