@@ -1,6 +1,7 @@
 ## Domain Registry Interface, DENIC policies
 ##
 ## Copyright (c) 2007,2008,2009 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>. All rights reserved.
+##           (c) 2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,12 +19,14 @@
 package Net::DRI::DRD::DENIC;
 
 use strict;
+use warnings;
+
 use base qw/Net::DRI::DRD/;
 
-use Net::DRI::DRD::ICANN;
 use DateTime::Duration;
+use Net::DRI::Util;
 
-our $VERSION=do { my @r=(q$Revision: 1.3 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.5 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -54,6 +57,7 @@ Tonnerre Lombard, E<lt>tonnerre.lombard@sygroup.chE<gt>
 =head1 COPYRIGHT
 
 Copyright (c) 2007,2008,2009 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
+          (c) 2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -81,56 +85,16 @@ sub periods  { return map { DateTime::Duration->new(years => $_) } (1..10); }
 sub name     { return 'DENIC'; }
 sub tlds     { return ('de','9.4.e164.arpa'); } ## *.9.4.e164.arpa can be queried over IRIS DCHK, do not know about RRI support
 sub object_types { return ('domain','contact'); }
-
-sub transport_protocol_compatible
-{
- my ($self,$to,$po)=@_;
- my $pn=$po->name();
- my $pv=$po->version();
- my $tn=$to->name();
-
- return 1 if (($pn eq 'RRI') && ($tn eq 'socket_inet'));
- return 1 if (($pn eq 'IRIS') && ($tn eq 'socket_inet'));
- return;
-}
-
-## TODO: see if that could be useful (such as being used by Net::DRI::Shell) ; if so propagate to other DRD modules
-sub available_profile_types { return qw/rri dchk/; }
+sub profile_types { return qw/rri dchk/; }
 
 sub transport_protocol_default
 {
- my ($drd,$ndr,$type,$ta,$pa)=@_;
- $type='rri' if (!defined($type) || ref($type));
- if ($type eq 'rri')
- {
-  return ('Net::DRI::Transport::Socket','Net::DRI::Protocol::RRI') unless (defined($ta) && defined($pa));
-  my %ta=( defer=>1,
-           close_after=>1,
-           socktype=>'tcp',
-           remote_host=>'rri.test.denic.de',
-           remote_port=>51131,
-           protocol_connection=>'Net::DRI::Protocol::RRI::Connection',
-           protocol_version=>'2.0',
-           (ref($ta) eq 'ARRAY')? %{$ta->[0]} : %$ta,
-         );
-  my @pa=(ref($pa) eq 'ARRAY' && @$pa)? @$pa : ('2.0');
-  return ('Net::DRI::Transport::Socket',[\%ta],'Net::DRI::Protocol::RRI',\@pa);
- }
- if ($type eq 'dchk')
- {
-  my %ta=( defer=>1,
-           socktype=>'udp',
-           find_remote_server => ['de.','DCHK1:iris.lwz'], ## authority / service
-           protocol_connection=>'Net::DRI::Protocol::IRIS::LWZ',
-           protocol_version=>'1.0',
-           timeout => 1, ## RFC4993 Section 4 gives recommandation for timeouts and retry algorithm
-           pause => 2,
-           retry => 5, ## computed so that the whole sequence stops after 60 seconds: t,p+2t,3/2(p+2)-2+4t,3/2*3/2*(p+2)-2+8t,...
-           (ref($ta) eq 'ARRAY')? %{$ta->[0]} : %$ta,
-         );
-  my @pa=(ref($pa) eq 'ARRAY' && @$pa)? @$pa : ('1.0','de'); ## (version,authority)
-  return ('Net::DRI::Transport::Socket',[\%ta],'Net::DRI::Protocol::IRIS',\@pa);
- }
+ my ($self,$type)=@_;
+
+ return ('Net::DRI::Transport::Socket',{remote_host=>'rri.test.denic.de',remote_port=>51131,defer=>1,close_after=>1,socktype=>'tcp'},'Net::DRI::Protocol::RRI',{version=>'2.0'}) if $type eq 'rri';
+ ## DENIC IRIS server does not support request deflation, see 3.1.7 "no-inflation-support-error", so we force no deflate
+ return ('Net::DRI::Transport::Socket',{find_remote_server => ['de.','DCHK1:iris.lwz']},'Net::DRI::Protocol::IRIS',{version=>'1.0',authority=>'de',request_deflate=>0})          if $type eq 'dchk';
+ return;
 }
 
 ####################################################################################################
@@ -165,6 +129,7 @@ sub contact_update
 sub domain_update
 {
  my ($self, $reg, $dom, $changes, $rd) = @_;
+ $rd=Net::DRI::Util::create_params('domain_update',$rd);
  my $cs = $reg->get_info('contact', 'domain', $dom);
  my $ns = $reg->get_info('ns', 'domain', $dom);
 
