@@ -1,6 +1,6 @@
 ## Domain Registry Interface, .LU policy from DocRegistrar-2.0.6.pdf
 ##
-## Copyright (c) 2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2007-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,12 +18,15 @@
 package Net::DRI::DRD::LU;
 
 use strict;
+use warnings;
+
 use base qw/Net::DRI::DRD/;
 
-our $VERSION=do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+use Net::DRI::Data::Contact::LU;
 
-__PACKAGE__->make_exception_for_unavailable_operations(qw/domain_renew domain_transfer_accept domain_transfer_refuse contact_transfer contact_transfer_start contact_transfer_stop
-contact_transfer_query contact_transfer_accept contact_transfer_refuse/);
+our $VERSION=do { my @r=(q$Revision: 1.6 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+
+__PACKAGE__->make_exception_for_unavailable_operations(qw/domain_renew domain_transfer_accept domain_transfer_refuse contact_transfer contact_transfer_start contact_transfer_stop contact_transfer_query contact_transfer_accept contact_transfer_refuse/);
 
 =pod
 
@@ -53,7 +56,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2007-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -81,24 +84,21 @@ sub periods  { return; } ## registry does not expect any duration at all
 sub name     { return 'DNSLU'; }
 sub tlds     { return ('lu'); }
 sub object_types { return ('domain','contact','ns'); }
-
-sub transport_protocol_compatible
-{
- my ($self,$to,$po)=@_;
- my $pn=$po->name();
- my $tn=$to->name();
-
- return 1 if (($pn eq 'EPP') && ($tn eq 'socket_inet'));
- return 1 if (($pn eq 'Whois') && ($tn eq 'socket_inet'));
- return;
-}
+sub profile_types { return qw/epp whois/; }
 
 sub transport_protocol_default
 {
- my ($drd,$ndr,$type,$ta,$pa)=@_;
- $type='epp' if (!defined($type) || ref($type));
- return Net::DRI::DRD::_transport_protocol_default_epp('Net::DRI::Protocol::EPP::Extensions::LU',$ta,$pa) if ($type eq 'epp');
- return ('Net::DRI::Transport::Socket',[{%Net::DRI::DRD::PROTOCOL_DEFAULT_WHOIS,remote_host=>'whois.dns.lu'}],'Net::DRI::Protocol::Whois',[]) if (lc($type) eq 'whois');
+ my ($self,$type)=@_;
+
+ return ('Net::DRI::Transport::Socket',{},'Net::DRI::Protocol::EPP::Extensions::LU',{})              if $type eq 'epp';
+ return ('Net::DRI::Transport::Socket',{remote_host=>'whois.dns.lu'},'Net::DRI::Protocol::Whois',{}) if $type eq 'whois';
+ return;
+}
+
+sub set_factories
+{
+ my ($self,$po)=@_;
+ $po->factories('contact',sub { return Net::DRI::Data::Contact::LU->new(@_); });
 }
 
 ####################################################################################################
@@ -134,70 +134,49 @@ sub domain_status_allows
 sub domain_restore
 {
  my ($self,$ndr,$domain)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'restore');
+ $self->enforce_domain_name_constraints($ndr,$domain,'restore');
  return $ndr->process('domain','restore',[$domain]);
 }
 
-sub domain_trade_start
-{
- my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'trade');
- return $ndr->process('domain','trade_request',[$domain,$rd]);
-}
-
-sub domain_trade_query
-{
- my ($self,$ndr,$domain)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'trade');
- return $ndr->process('domain','trade_query',[$domain]);
-}
-
-sub domain_trade_stop
-{
- my ($self,$ndr,$domain)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'trade');
- return $ndr->process('domain','trade_cancel',[$domain]);
-}
-
 sub domain_transfer_trade_start
 {
  my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'transfer_trade');
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_trade');
  return $ndr->process('domain','transfer_trade_request',[$domain,$rd]);
 }
 
 sub domain_transfer_trade_query
 {
  my ($self,$ndr,$domain)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'transfer_trade');
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_trade');
  return $ndr->process('domain','transfer_trade_query',[$domain]);
 }
 
 sub domain_transfer_trade_stop
 {
  my ($self,$ndr,$domain)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'transfer_trade');
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_trade');
  return $ndr->process('domain','transfer_trade_cancel',[$domain]);
 }
 
 sub domain_transfer_restore_start
 {
  my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'transfer_restore');
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_restore');
  return $ndr->process('domain','transfer_restore_request',[$domain,$rd]);
 }
 
 sub domain_transfer_restore_query
 {
  my ($self,$ndr,$domain)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'transfer_restore');
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_restore');
  return $ndr->process('domain','transfer_restore_query',[$domain]);
 }
 
 sub domain_transfer_restore_stop
 {
  my ($self,$ndr,$domain)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'transfer_restore');
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_restore');
  return $ndr->process('domain','transfer_restore_cancel',[$domain]);
 }
 
