@@ -1,6 +1,6 @@
 ## Domain Registry Interface, .NO policies for Net::DRI
 ##
-## Copyright (c) 2008,2009 UNINETT Norid AS, E<lt>http://www.norid.noE<gt>,
+## Copyright (c) 2008-2010 UNINETT Norid AS, E<lt>http://www.norid.noE<gt>,
 ##                    Trond Haugen E<lt>info@norid.noE<gt>
 ##                    All rights reserved.
 ##
@@ -20,13 +20,15 @@
 package Net::DRI::DRD::NO;
 
 use strict;
+use warnings;
+
 use base qw/Net::DRI::DRD/;
 
 use DateTime::Duration;
 use Net::DRI::Util;
 use Net::DRI::Exception;
 
-our $VERSION = do { my @r = ( q$Revision: 1.3 $ =~ /\d+/gxm ); sprintf( "%d" . ".%02d" x $#r, @r ); };
+our $VERSION = do { my @r = ( q$Revision: 1.6 $ =~ /\d+/gxm ); sprintf( "%d" . ".%02d" x $#r, @r ); };
 
 # let contact check support be decided by the server policy
 __PACKAGE__->make_exception_for_unavailable_operations(qw/domain_transfer_accept domain_transfer_refuse contact_transfer_stop contact_transfer_query contact_transfer_accept contact_transfer_refuse/);
@@ -59,7 +61,7 @@ Trond Haugen E<lt>info@norid.noE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008,2009 UNINETT Norid AS, E<lt>http://www.norid.noE<gt>,
+Copyright (c) 2008-2010 UNINETT Norid AS, E<lt>http://www.norid.noE<gt>,
 Trond Haugen E<lt>info@norid.noE<gt>
 All rights reserved.
 
@@ -90,46 +92,19 @@ sub periods {
 sub name         { return 'NORID'; }
 sub tlds         { return ('NO'); }
 sub object_types { return ( 'domain', 'contact', 'ns' ); }
-
-sub transport_protocol_compatible {
-    my ( $self, $to, $po ) = @_;
-    my $pn = $po->name();
-    my $tn = $to->name();
-
-    return 1 if ( ( $pn eq 'EPP' ) && ( $tn eq 'socket_inet' ) );
-    return;
-}
+sub profile_types { return qw/epp/; }
 
 sub transport_protocol_default {
-    my ( $drd, $ndr, $type, $ta, $pa ) = @_;
-    $type = 'epp' if ( !defined($type) || ref($type) );
-    return Net::DRI::DRD::_transport_protocol_default_epp(
-        'Net::DRI::Protocol::EPP::Extensions::NO',
-        $ta, $pa )
-        if ( $type eq 'epp' );
+    my ($self,$type)=@_;
 
+    return ('Net::DRI::Transport::Socket',{},'Net::DRI::Protocol::EPP::Extensions::NO',{}) if $type eq 'epp';
 # suppress until whois is supported
-#return ('Net::DRI::Transport::Socket',[{%Net::DRI::DRD::PROTOCOL_DEFAULT_WHOIS,remote_host=>'whois.norid.no'}],'Net::DRI::Protocol::Whois',[]) if (lc($type) eq 'whois');
+#return ('Net::DRI::Transport::Socket',{remote_host=>'whois.norid.no'},'Net::DRI::Protocol::Whois',{}) if $type eq 'whois';
 
     return;
 }
 
-#########################################################################
-# need to accept all tlds for hosts, so subclass this method and
-# remove the tld check
-sub verify_name_host {
-    my ( $self, $ndr, $host, $checktld ) = @_;
-    $checktld ||= 0;
-    ( $host, $checktld ) = ( $ndr, $host )
-        unless ( defined($ndr)
-        && $ndr
-        && ( ref($ndr) eq 'Net::DRI::Registry' ) );
-
-    $host = $host->get_names(1) if ( ref($host) );
-    my $r = $self->check_name($host);
-    return $r if ($r);
-    return 0;
-}
+####################################################################################################
 
 =head1 verify_name_domain
 
@@ -141,7 +116,7 @@ in 'check_name'.
 
 However, we do not subclass the 'is_my_tld' as .AT has done,
 but we then have to call it in a non-strict mode to allow for
-domain names with multiple lables.
+domain names with multiple labels.
 
 The combination should then allow multiple labels and also
 to use CC-codes in lables, like 'se.vgs.no'
@@ -158,10 +133,6 @@ sub verify_name_domain
 
 sub verify_duration_renew {
     my ( $self, $ndr, $duration, $domain, $curexp ) = @_;
-    ( $duration, $domain, $curexp ) = ( $ndr, $duration, $domain )
-        unless ( defined($ndr)
-        && $ndr
-        && ( ref($ndr) eq 'Net::DRI::Registry' ) );
 
     if ( defined($duration) ) {
         my ( $y, $m ) = $duration->in_units( 'years', 'months' );
@@ -170,9 +141,6 @@ sub verify_duration_renew {
         unless ( ( $y == 1 && $m == 0 )
             || ( $y == 0 && ( $m >= 1 && $m <= 12 ) ) )
         {
-            Net::DRI::Exception::usererr_invalid_parameters(
-                'Invalid duration for renew/transfer_execute, must be 1..12 months'
-            );
             return 1;    # if exception is removed, return an error
         }
     }
@@ -190,9 +158,9 @@ sub domain_operation_needs_is_mine {
 
 sub domain_withdraw {
     my ( $self, $ndr, $domain, $rd ) = @_;
-    $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'withdraw');
+    $self->enforce_domain_name_constraints($ndr,$domain,'withdraw');
 
-    $rd = {} unless ( defined($rd) && ( ref($rd) eq 'HASH' ) );
+    $rd=Net::DRI::Util::create_params('domain_withdraw',$rd);
     $rd->{transactionname} = 'withdraw';
 
     my $rc = $ndr->process( 'domain', 'withdraw', [ $domain, $rd ] );
@@ -202,9 +170,9 @@ sub domain_withdraw {
 sub domain_transfer_execute
 {
  my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($ndr,$domain,'transfer_execute');
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_execute');
 
- $rd={} unless (defined($rd) && (ref($rd) eq 'HASH'));
+ $rd=Net::DRI::Util::create_params('domain_transfer_execute',$rd);
  $rd->{transactionname} = 'transfer_execute';
 
  my $rc=$ndr->process('domain','transfer_execute',[$domain,$rd]);
@@ -217,17 +185,14 @@ sub host_update {
     my ( $self, $ndr, $dh, $tochange, $rh ) = @_;
     my $fp = $ndr->protocol->nameversion();
 
-    my $name
-        = ( UNIVERSAL::isa( $dh, 'Net::DRI::Data::Hosts' ) )
-        ? $dh->get_details(1)
-        : $dh;
-    $self->err_invalid_host_name($name) if $self->verify_name_host($name);
+    my $name=Net::DRI::Util::is_class($dh,'Net::DRI::Data::Hosts') ? $dh->get_details(1) : $dh;
+    $self->enforce_host_name_constraints($ndr,$name);
     Net::DRI::Util::check_isa( $tochange, 'Net::DRI::Data::Changes' );
 
     foreach my $t ( $tochange->types() ) {
         Net::DRI::Exception->die( 0, 'DRD', 6,
             "Change host_update/${t} not handled" )
-            unless ( $t =~ m/^(?:ip|status|name|contact)$/mx );
+            unless ( $t =~ m/^(?:ip|status|name|contact|facets)$/mx );
         next if $ndr->protocol_capable( 'host_update', $t );
         Net::DRI::Exception->die( 0, 'DRD', 5,
             "Protocol ${fp} is not capable of host_update/${t}" );
@@ -245,7 +210,7 @@ sub host_update {
         Net::DRI::Util::check_isa( $_, 'Net::DRI::Data::StatusList' );
     }
     foreach ( @{ $what{name} } ) {
-        $self->err_invalid_host_name($_) if $self->verify_name_host($_);
+	$self->enforce_host_name_constraints($ndr,$_);
     }
 
     foreach my $w ( keys(%what) ) {
@@ -278,28 +243,34 @@ sub host_update {
 }
 
 sub message_retrieve {
-    my ( $self, $ndr, $id ) = @_;
-    my $rc = $ndr->process( 'message', 'noretrieve', [$id] );
+    my ( $self, $ndr, $rd ) = @_;
+
+    my $rc = $ndr->process( 'message', 'noretrieve', [$rd] );
     return $rc;
 }
 
 sub message_delete {
-    my ( $self, $ndr, $id ) = @_;
-    my $rc = $ndr->process( 'message', 'nodelete', [$id] );
+    my ( $self, $ndr, $id, $rd ) = @_;
+
+    my $rc = $ndr->process( 'message', 'nodelete', [$id, $rd] );
     return $rc;
 }
 
 sub message_waiting {
-    my ( $self, $ndr ) = @_;
-    my $c = $self->message_count($ndr);
+    my ( $self, $ndr, $rd ) = @_;
+
+    my $c = $self->message_count($ndr, $rd);
     return ( defined($c) && $c ) ? 1 : 0;
 }
 
 sub message_count {
-    my ( $self, $ndr ) = @_;
+    my ( $self, $ndr, $rd ) = @_;
+
     my $count = $ndr->get_info( 'count', 'message', 'info' );
     return $count if defined($count);
-    my $rc = $ndr->process( 'message', 'noretrieve' );
+
+    my $rc = $ndr->process( 'message', 'noretrieve',  [$rd] );
+
     return unless $rc->is_success();
     $count = $ndr->get_info( 'count', 'message', 'info' );
     return ( defined($count) && $count ) ? $count : 0;
