@@ -1,6 +1,6 @@
 ## Domain Registry Interface, .UK (Nominet) policies for Net::DRI
 ##
-## Copyright (c) 2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2007-2011 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,6 +18,8 @@
 package Net::DRI::DRD::Nominet;
 
 use strict;
+use warnings;
+
 use base qw/Net::DRI::DRD/;
 
 use Net::DRI::Util;
@@ -25,14 +27,14 @@ use Net::DRI::Exception;
 
 use DateTime::Duration;
 
-our $VERSION=do { my @r=(q$Revision: 1.6 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.10 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 ## No status at all with Nominet
 ## Only domain:check is available
 ## Only domain transfer op=req and refuse/accept
 ## The delete command applies only to domain names.  Accounts, contacts and nameservers cannot be explicitly deleted, but are automatically deleted when no longer referenced.
 ## No direct contact/host create
-__PACKAGE__->make_exception_for_unavailable_operations(qw/domain_update_status_add domain_update_status_del domain_update_status_set domain_update_status domain_status_allows_delete domain_status_allows_update domain_status_allows_transfer domain_status_allows_renew domain_status_allows domain_current_status host_update_status_add host_update_status_del host_update_status_set host_update_status host_current_status contact_update_status_add contact_update_status_del contact_update_status_set contact_update_status contact_current_status host_check host_check_multi host_exist contact_check contact_check_multi contact_exist contact_transfer contact_transfer_start contact_transfer_stop contact_transfer_query contact_transfer_accept contact_transfer_refuse domain_transfer_stop domain_transfer_query host_delete contact_delete host_create contact_create/);
+__PACKAGE__->make_exception_for_unavailable_operations(qw/domain_update_status_add domain_update_status_del domain_update_status_set domain_update_status domain_status_allows_delete domain_status_allows_update domain_status_allows_transfer domain_status_allows_renew domain_status_allows domain_current_status host_update_status_add host_update_status_del host_update_status_set host_update_status host_current_status contact_update_status_add contact_update_status_del contact_update_status_set contact_update_status contact_current_status host_check host_exist contact_check contact_exist contact_transfer contact_transfer_start contact_transfer_stop contact_transfer_query contact_transfer_accept contact_transfer_refuse domain_transfer_stop domain_transfer_query host_delete contact_delete host_create contact_create/);
 
 =pod
 
@@ -62,7 +64,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2007-2011 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -80,7 +82,7 @@ sub new
 {
  my $class=shift;
  my $self=$class->SUPER::new(@_);
- $self->{info}->{host_as_attr}=0;
+ $self->{info}->{host_as_attr}=1;
 
  bless($self,$class);
  return $self;
@@ -90,49 +92,24 @@ sub periods  { return map { DateTime::Duration->new(years => $_) } (2); }
 sub name     { return 'Nominet'; }
 sub tlds     { return qw/co.uk ltd.uk me.uk net.uk org.uk plc.uk sch.uk/; } ##See http://www.nominet.org.uk/registrants/aboutdomainnames/rules/
 sub object_types { return ('domain','contact','ns','account'); }
-
-sub transport_protocol_compatible
-{
- my ($self,$to,$po)=@_;
- my $pn=$po->name();
- my $tn=$to->name();
-
- return 1 if (($pn eq 'EPP') && ($tn eq 'socket_inet'));
-## return 1 if (($pn eq 'DAS') && ($tn eq 'socket_inet'));
- return;
-}
+sub profile_types { return qw/epp epp_nominet epp_standard/; }
 
 sub transport_protocol_default
 {
- my ($drd,$ndr,$type,$ta,$pa)=@_;
- $type='epp' if (!defined($type) || ref($type));
- if ($type eq 'epp')
- {
-  return ('Net::DRI::Transport::Socket','Net::DRI::Protocol::EPP::Extensions::Nominet') unless (defined($ta) && defined($pa));
-  my %ta=(	%Net::DRI::DRD::PROTOCOL_DEFAULT_EPP,
-		remote_host => 'epp.nominet.org.uk',
-                protocol_data => { login_service_filter => \&_filter_objuri },
-		(ref($ta) eq 'ARRAY')? %{$ta->[0]} : %$ta,
-	);
-  my @pa=(ref($pa) eq 'ARRAY' && @$pa)? @$pa : ('1.0');
-  $ta{client_login}='#'.$ta{client_login} if (defined($ta{client_login}) && length($ta{client_login})==2); # as seen on http://www.nominet.org.uk/registrars/systems/epp/login/
-  return ('Net::DRI::Transport::Socket',[\%ta],'Net::DRI::Protocol::EPP::Extensions::Nominet',\@pa);
- }
-## return ('Net::DRI::Transport::Socket',[{%Net::DRI::DRD::PROTOCOL_DEFAULT_DAS,remote_host=>'dac.nic.uk',remote_port=>2043,close_after=>0}],'Net::DRI::Protocol::DAS::Nominet',[]) if (lc($type) eq 'das');
+ my ($self,$type)=@_;
+
+ return ('Net::DRI::Transport::Socket',{remote_host => 'epp.nominet.org.uk'},'Net::DRI::Protocol::EPP::Extensions::Nominet',{}) if ($type eq 'epp' || $type eq 'epp_nominet');
+ return ('Net::DRI::Transport::Socket',{remote_host => 'epp.nominet.org.uk'},'Net::DRI::Protocol::EPP',{})                      if ($type eq 'epp_standard');
+ return;
 }
 
-## The registry gives back both -1.0 and -1.1 version of its namespaces, we keep only the latest
-## See http://www.nominet.org.uk/registrars/systems/epp/Namespace+URIs/
-sub _filter_objuri
+sub transport_protocol_init
 {
- my @a;
- foreach my $a (@_)
- {
-  my ($t,$v)=@$a;
-  next if ($t eq 'objURI' && $v=~m/nom-\S+-1\.0$/);
-  push @a,$a;
- }
- return @a;
+ my ($self,$type,$tc,$tp,$pc,$pp,$test)=@_;
+
+ ## As seen on http://www.nominet.org.uk/registrars/systems/nominetepp/login/
+ $tp->{client_login}='#'.$tp->{client_login} if ($type eq 'epp' && defined $tp->{client_login} && length $tp->{client_login}==2);
+ return;
 }
 
 ####################################################################################################
@@ -141,16 +118,13 @@ sub _filter_objuri
 sub verify_duration_renew
 {
  my ($self,$ndr,$duration,$domain,$curexp)=@_;
- ($duration,$domain,$curexp)=($ndr,$duration,$domain) unless (defined($ndr) && $ndr && (ref($ndr) eq 'Net::DRI::Registry'));
 
 ## +Renew commands will only be processed if the expiry date of the domain name is within 6 months.
 
- if (defined($duration))
- {
-  my ($y,$m)=$duration->in_units('years','months');
-  return 1 unless ($y==2 && $m==0); ## Only 24m or 2y allowed
- }
+ return 0 unless defined $duration;
 
+ my ($y,$m)=$duration->in_units('years','months');
+ return 1 unless ($y==2 && $m==0); ## Only 24m or 2y allowed
  return 0; ## everything ok
 }
 
@@ -173,7 +147,7 @@ sub host_update
  my $fp=$ndr->protocol->nameversion();
 
  my $name=Net::DRI::Util::isa_hosts($dh)? $dh->get_details(1) : $dh;
- $self->err_invalid_host_name($name) if $self->verify_name_host($name);
+ $self->enforce_host_name_constraints($ndr,$name);
  Net::DRI::Util::check_isa($tochange,'Net::DRI::Data::Changes');
 
  foreach my $t ($tochange->types())
@@ -187,7 +161,7 @@ sub host_update
            'name'   => [ $tochange->all_defined('name') ],
           );
  foreach (@{$what{ip}})     { Net::DRI::Util::check_isa($_,'Net::DRI::Data::Hosts'); }
- foreach (@{$what{name}})   { $self->err_invalid_host_name($_) if $self->verify_name_host($_); }
+ foreach (@{$what{name}})   { $self->enforce_host_name_constraints($ndr,$_); }
 
  foreach my $w (keys(%what))
  {
@@ -235,5 +209,20 @@ sub account_merge
  return $ndr->process('account','merge',[$c,$cs]);
 }
 
+sub domain_unrenew
+{
+ my ($self,$ndr,$domain,$rd)=@_;
+ $self->enforce_domain_name_constraints($ndr,$domain,'unrenew');
+ return $ndr->process('domain','unrenew',[$domain,$rd]);
+}
+
+sub account_list_domains
+{
+ my ($self,$ndr,$rd,$rh)=@_;
+ my $rc=$ndr->try_restore_from_cache('account','domains','list');
+ if (! defined $rc) { $rc=$ndr->process('account','list_domains',[$rd,$rh]); }
+ return $rc;
+}
+
 ####################################################################################################
 1;
