@@ -1,6 +1,6 @@
-## Domain Registry Interface, ``Verisign Naming and Directory Services'' Registry Driver for .COM & .NET
+## Domain Registry Interface, "Verisign Naming and Directory Services" Registry Driver for .COM .NET .CC .TV .BZ .JOBS
 ##
-## Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,19 +18,19 @@
 package Net::DRI::DRD::VNDS;
 
 use strict;
+use warnings;
+
 use base qw/Net::DRI::DRD/;
 
-use Net::DRI::DRD::ICANN;
 use DateTime::Duration;
-use DateTime;
 
-our $VERSION=do { my @r=(q$Revision: 1.18 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.20 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
 =head1 NAME
 
-Net::DRI::DRD::VNDS - Verisign .COM/.NET Registry driver for Net::DRI
+Net::DRI::DRD::VNDS - Verisign .COM/.NET/.CC/.TV/.BZ/.JOBS Registry driver for Net::DRI
 
 =head1 DESCRIPTION
 
@@ -54,7 +54,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -68,29 +68,29 @@ See the LICENSE file that comes with this distribution for more details.
 
 ####################################################################################################
 
-sub periods      { return map { DateTime::Duration->new(years => $_) } (1..10); }
-sub name         { return 'VNDS'; }
-sub tlds         { return ('com','net','cc','tv','bz','jobs'); } ## If this changes, VeriSign/NameStore will need to be updated also
-sub object_types { return ('domain','ns'); }
-
-sub transport_protocol_compatible 
+sub new
 {
- my ($self,$to,$po)=@_;
- my $pn=$po->name();
- my $pv=$po->version();
- my $tn=$to->name();
-
- return 1 if (($pn eq 'EPP') && ($tn eq 'socket_inet'));
- return 1 if (($pn eq 'Whois') && ($tn eq 'socket_inet'));
- return;
+ my $class=shift;
+ my $self=$class->SUPER::new(@_);
+ $self->{info}->{check_limit}=5;
+ bless($self,$class);
+ return $self;
 }
 
+sub periods       { return map { DateTime::Duration->new(years => $_) } (1..10); }
+sub name          { return 'VNDS'; }
+sub tlds          { return qw/com net cc tv bz jobs/; } ## If this changes, VeriSign/NameStore will need to be updated also
+sub object_types  { return qw/domain ns/; }
+sub profile_types { return qw/epp whois/; }
+
 sub transport_protocol_default
 {
- my ($drd,$ndr,$type,$ta,$pa)=@_;
- $type='epp' if (!defined($type) || ref($type));
- return Net::DRI::DRD::_transport_protocol_default_epp('Net::DRI::Protocol::EPP::Extensions::VeriSign',$ta,$pa) if ($type eq 'epp');
- return ('Net::DRI::Transport::Socket',[{%Net::DRI::DRD::PROTOCOL_DEFAULT_WHOIS,remote_host=>'whois.verisign-grs.com'}],'Net::DRI::Protocol::Whois',[]) if (lc($type) eq 'whois');
+ my ($self,$type)=@_;
+
+ return ('Net::DRI::Transport::Socket',{},'Net::DRI::Protocol::RRP',{})                                        if $type eq 'rrp'; ## this is used only for internal tests
+ return ('Net::DRI::Transport::Socket',{},'Net::DRI::Protocol::EPP::Extensions::VeriSign',{})                  if $type eq 'epp';
+ return ('Net::DRI::Transport::Socket',{remote_host=>'whois.verisign-grs.com'},'Net::DRI::Protocol::Whois',{}) if $type eq 'whois';
+ return;
 }
 
 ####################################################################################################
@@ -108,18 +108,30 @@ sub verify_name_domain
 sub verify_duration_transfer
 {
  my ($self,$ndr,$duration,$domain,$op)=@_;
- ($duration,$domain,$op)=($ndr,$duration,$domain) unless (defined($ndr) && $ndr && (ref($ndr) eq 'Net::DRI::Registry'));
 
- return 0 unless ($op eq 'start'); ## we are not interested by other cases, they are always OK
- my $rc=$self->domain_info($ndr,$domain,{hosts=>'none'});
- return 1 unless ($rc->is_success());
- my $trdate=$ndr->get_info('trDate');
- return 0 unless ($trdate && $trdate->isa('DateTime'));
- 
- my $now=DateTime->now(time_zone => $trdate->time_zone()->name());
- my $cmp=DateTime->compare($now,$trdate+DateTime::Duration->new(days => 15));
- return ($cmp == 1)? 0 : 1; ## we must have : now > transferdate + 15days
- ## we return 0 if OK, anything else if not
+ return $self->_verify_duration_transfer_15days($ndr,$duration,$domain,$op);
+}
+
+####################################################################################################
+
+sub domain_whowas
+{
+ my ($self,$ndr,$domain,$rd)=@_;
+ $self->enforce_domain_name_constraints($ndr,$domain,'whowas');
+
+ my $rc=$ndr->try_restore_from_cache('domain',$domain,'whowas');
+ if (! defined $rc) { $rc=$ndr->process('domain','whowas',[$domain,$rd]); }
+ return $rc;
+}
+
+sub domain_suggest
+{
+ my ($self,$ndr,$domain,$rd)=@_;
+ Net::DRI::Exception::usererr_invalid_parameters('domain_suggestion domain/key must be a string of 2 to 32 characters') unless Net::DRI::Util::xml_is_string($domain,2,32);
+ $self->enforce_domain_name_constraints($ndr,$domain,'suggestion') unless $domain=~m/\s/; ## if we have a space, then $domain is a list of keywords
+ my $rc=$ndr->try_restore_from_cache('domain',$domain,'suggesion');
+ if (! defined $rc) { $rc=$ndr->process('domain','suggestion',[$domain,$rd]); }
+ return $rc;
 }
 
 ####################################################################################################
