@@ -1,6 +1,6 @@
 ## Domain Registry Interface, Handling of contact data
 ##
-## Copyright (c) 2005,2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,6 +18,8 @@
 package Net::DRI::Data::Contact;
 
 use strict;
+use warnings;
+
 use base qw(Class::Accessor::Chained); ## provides a new() method
 
 our @ATTRS=qw(name org street city sp pc cc email voice fax loid roid srid auth disclose);
@@ -29,7 +31,7 @@ use Net::DRI::Util;
 use Email::Valid;
 use Encode (); ## we need here direct use of Encode, not through Net::DRI::Util::encode_* as we need the default substitution for unknown data
 
-our $VERSION=do { my @r=(q$Revision: 1.12 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.15 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -52,12 +54,14 @@ EPP allows a localized form (content is in unrestricted UTF-8) and international
 (content MUST be represented in a subset of UTF-8 that can be represented 
 in the 7-bit US-ASCII character set). Not all registries support both forms.
 
-When setting values, you pass one element if both forms are equal or two elements
-as a list (first the localized form, then the internationalized one).
+When setting values, you pass wo elements as a list (first the localized form, 
+then the internationalized one), or only one element that will be taken as the localized form.
 When getting values, in list context you get back both values, in scalar context you get
 back the first one, that is the localized form.
 
 You can also use methods int2loc() and loc2int() to create one version from the other.
+These 2 methods may be used automatically inside Net::DRI as needed, depending on what
+the registry expects and the operation conducted (like a contact create).
 
 =head1 METHODS
 
@@ -69,11 +73,11 @@ local object ID for this contact, never sent to registry (can be used to track t
 
 server ID, ID of the object as known by the registry in which it was created
 
-=head2 id() 
+=head2 id()
 
 an alias (needed for Net::DRI::Data::ContactSet) of the previous method
 
-=head2 roid() 
+=head2 roid()
 
 registry/remote object id (internal to a registry)
 
@@ -101,7 +105,7 @@ state/province of the contact
 
 postal code of the contact
 
-=head2 cc() 
+=head2 cc()
 
 alpha2 country code of the contact (will be verified against list of valid country codes)
 
@@ -119,7 +123,7 @@ fax number of the contact (same form as above)
 
 =head2 auth()
 
-authentification for this contact (hash ref with a key 'pw' and a value being the password)
+authentication for this contact (hash ref with a key 'pw' and a value being the password)
 
 =head2 disclose()
 
@@ -162,7 +166,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -193,25 +197,32 @@ sub attributes
  my $class=shift;
  $class=ref($class) || $class;
  no strict 'refs'; ## no critic (ProhibitNoStrict)
- return @{${$class.'::ATTRS'}}; 
+ return @{${$class.'::ATTRS'}};
 }
 
 ## Overrides method in Class::Accessor, needed for int/loc data
 sub get
 {
  my ($self,$what)=@_;
- return unless defined($what) && $what && exists($self->{$what});
+ return unless defined $what && $what && exists $self->{$what};
  my $d=$self->{$what};
  return $d unless ($what=~m/^(name|org|street|city|sp|pc|cc)$/);
- if ($what eq 'street') ## special case because it is always a ref array
+
+ ## Special case for street because it is always a ref array, but a complicate one, we have either
+ ## [ X, Y, Z ] (with Y and/or Z optional)
+ ## [ undef, [ X, Y, Z ] ]
+ ## [ [ X, Y, Z ] , undef ]
+ ## [ [ X, Y, Z ], [ XX, YY, ZZ ] ]
+ ## [ undef, undef ]
+ if ($what eq 'street')
  {
-  return $d if !ref($d); ## should not happen, since it is either a ref array of up to 3 elements, or a ref array of two such ref arrays
-  return $d if !ref($d->[0]);
+  Net::DRI::Exception::usererr_invalid_parameters('Invalid street information, should be one or two ref arrays of up to 3 elements each') unless ref $d eq 'ARRAY';
+  return wantarray ? ($d, undef) : $d unless 2==grep { ! defined $_ || ref $_ eq 'ARRAY' } @$d;
  } else
  {
-  return $d if !ref($d);
+  return $d unless ref $d eq 'ARRAY';
  }
- return wantarray()? @$d : $d->[0];
+ return wantarray ? @$d : $d->[0];
 }
 
 sub loc2int
@@ -220,11 +231,17 @@ sub loc2int
  foreach my $f (qw/name org city sp pc cc/)
  {
   my @c=$self->$f();
-  $c[1]=defined($c[0])? Encode::encode('ascii',$c[0],0) : undef;
+  $c[1]=defined $c[0] ? Encode::encode('ascii',$c[0],0) : undef;
   $self->$f(@c);
  }
  my @c=$self->street();
- $c[1]=[ map { defined($_)? Encode::encode('ascii',$_,0) : undef } defined($c[0])? @{$c[0]} : () ];
+ if (defined $c[0])
+ {
+  $c[1]=[ map { defined $_ ? Encode::encode('ascii',$_,0) : undef } @{$c[0]} ];
+ } else
+ {
+  $c[1]=$c[0]=[];
+ }
  $self->street(@c);
  return $self;
 }
@@ -242,14 +259,14 @@ sub int2loc
 }
 
 sub has_loc { return shift->_has(0); }
-sub has_int  { return shift->_has(1); }
+sub has_int { return shift->_has(1); }
 sub _has
 {
  my ($self,$pos)=@_;
  my @d=map { ($self->$_())[$pos] } qw/name org city sp pc cc/;
  my $s=($self->street())[$pos];
- push @d,@$s if (defined($s) && ref($s));
- return (grep { defined} @d)? 1 : 0;
+ push @d,@$s if defined $s && ref $s eq 'ARRAY';
+ return (grep { defined } @d)? 1 : 0;
 }
 
 sub validate ## See RFC4933,4
@@ -260,19 +277,19 @@ sub validate ## See RFC4933,4
 
  if (!$change)
  {
-  Net::DRI::Exception::usererr_insufficient_parameters('Invalid contact information: name/city/cc/email/auth/srid mandatory') unless (scalar($self->name()) && scalar($self->city()) && scalar($self->cc()) && $self->email() && $self->auth() && $self->srid());
+  my @missing=grep { my $r=scalar $self->$_(); (defined $r && length $r)? 0 : 1 } qw/name city cc email auth srid/;
+  Net::DRI::Exception::usererr_insufficient_parameters('Mandatory contact information missing: '.join('/',@missing)) if @missing;
   push @errs,'srid' unless Net::DRI::Util::xml_is_token($self->srid(),3,16);
  }
 
- push @errs,'roid' if ($self->roid() && $self->roid()!~m/^\w{1,80}-\w{1,8}$/); ## \w includes _ in Perl
- 
+ push @errs,'srid' if ($self->srid() && ! Net::DRI::Util::xml_is_token($self->srid(),3,16));
  push @errs,'name' if ($self->name() && grep { !Net::DRI::Util::xml_is_normalizedstring($_,1,255) }     ($self->name()));
  push @errs,'org'  if ($self->org()  && grep { !Net::DRI::Util::xml_is_normalizedstring($_,undef,255) } ($self->org()));
 
  my @rs=($self->street());
  foreach my $i (0,1)
  {
-  next unless $rs[$i];
+  next unless defined $rs[$i];
   push @errs,'street' if ((ref($rs[$i]) ne 'ARRAY') || (@{$rs[$i]} > 3) || (grep { !Net::DRI::Util::xml_is_normalizedstring($_,undef,255) } @{$rs[$i]}));
  }
 
@@ -282,9 +299,9 @@ sub validate ## See RFC4933,4
  push @errs,'cc'   if ($self->cc()   && grep { !Net::DRI::Util::xml_is_token($_,2,2) }                  ($self->cc()));
  push @errs,'cc'   if ($self->cc()   && grep { !exists($Net::DRI::Util::CCA2{uc($_)}) }                 ($self->cc()));
 
- push @errs,'voice' if ($self->voice() && !Net::DRI::Util::xml_is_token($self->voice(),undef,17) && $self->voice()!~m/^\+[0-9]{1,3}\.[0-9]{1,14}(?:x\d+)?$/);
- push @errs,'fax'   if ($self->fax()   && !Net::DRI::Util::xml_is_token($self->fax(),undef,17)   && $self->fax()!~m/^\+[0-9]{1,3}\.[0-9]{1,14}(?:x\d+)?$/);
- push @errs,'email' if ($self->email() && !Net::DRI::Util::xml_is_token($self->email(),1,undef) && !Email::Valid->rfc822($self->email()));
+ push @errs,'voice' if ($self->voice() && ! ($self->voice()=~m/^\+[0-9]{1,3}\.[0-9]{1,14}(?:x\d+)?$/));
+ push @errs,'fax'   if ($self->fax()   && ! ($self->fax()=~m/^\+[0-9]{1,3}\.[0-9]{1,14}(?:x\d+)?$/));
+ push @errs,'email' if ($self->email() && ! (Net::DRI::Util::xml_is_token($self->email(),1,undef)  && Email::Valid->rfc822($self->email())));
 
  my $ra=$self->auth();
  push @errs,'auth' if ($ra && (ref($ra) eq 'HASH') && exists($ra->{pw}) && !Net::DRI::Util::xml_is_normalizedstring($ra->{pw}));
@@ -326,5 +343,12 @@ sub as_string
  return '('.$c.') '.join($sep,@v);
 }
 
+sub clone
+{
+ my ($self)=@_;
+ my $new=Net::DRI::Util::deepcopy($self);
+ return $new;
+}
+
 ####################################################################################################
 1;
