@@ -1,6 +1,6 @@
 ## Domain Registry Interface, Logging operations for Net::DRI
 ##
-## Copyright (c) 2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2009,2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -24,7 +24,7 @@ use base qw/Net::DRI::BaseClass/;
 use Net::DRI::Util;
 use Net::DRI::Exception;
 
-our $VERSION=do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf '%d'.('.%02d' x $#r), @r; };
+our $VERSION=do { my @r=(q$Revision: 1.5 $=~/\d+/g); sprintf '%d'.('.%02d' x $#r), @r; };
 
 __PACKAGE__->make_exception_if_not_implemented(qw/name setup_channel output/);
 
@@ -43,6 +43,7 @@ sub new
  if (! exists $self->{encoding}         || ! defined $self->{encoding} )         { $self->{encoding}        ='UTF-8'; }
  if (! exists $self->{format_header}    || ! defined $self->{format_header} )    { $self->{format_header}   ='%FULLTIME [%ULEVEL] <%TYPE>'; }
  if (! exists $self->{format_transport} || ! defined $self->{format_transport} ) { $self->{format_transport}='%TRID %UDIRECTION %MESSAGE'; }
+ if (! exists $self->{format_protocol}  || ! defined $self->{format_protocol} )  { $self->{format_protocol} ='%TRID %UDIRECTION %MESSAGE'; }
  bless $self,$c;
  $self->level($self->{level}); ## convert the level token to a numerical value
  return $self;
@@ -67,8 +68,7 @@ sub string_header
 {
  my ($self,$level,$type,$data)=@_;
  my $f=$self->{format_header};
- my $t=Net::DRI::Util::fulltime();
- $f=~s/%FULLTIME/$t/g;
+ $f=~s/%FULLTIME/Net::DRI::Util::fulltime()/eg;
  $f=~s/%ULEVEL/uc($level)/eg;
  $f=~s/%TYPE/$type/g;
  return $f;
@@ -88,7 +88,7 @@ sub string_data
    $ct=$msg->hint();
   } else
   {
-   if (! $msg->can('as_string')) { Net::DRI::Exception::err_method_not_implemented('as_string in '.ref $msg); }
+   if (! $msg->can('as_string')) { Net::DRI::Exception::method_not_implemented('as_string',ref $msg); }
   }
   $msg=$msg->as_string();
  }
@@ -110,14 +110,14 @@ sub string_data
   }
  }
 
- $data->{all}=join q{ },map { $_.q{=}.$data->{$_} } sort { $a cmp $b } keys %{$data}; ## this should be handy during debugging
+ $data->{all}=join q{ },map { $_.q{=}.(defined $data->{$_} ? $data->{$_} : '') } sort { $a cmp $b } keys %{$data}; ## this should be handy during debugging
  if (exists $data->{direction}) { $data->{udirection}=uc $data->{direction}; $data->{adirection}=$data->{direction} eq 'in'? 'C<=S' : 'C=>S';}
  my @r;
  foreach my $l (split /\n/,$msg)
  {
   my $f=$hdr.q{ }.$self->{'format_'.$type};
   $data->{message}=$l;
-  $f=~s/%([A-Z]+)/$data->{lc $1}/eg;
+  $f=~s/%([A-Z]+)/$data->{lc $1} || ''/eg;
   push @r,$f;
  }
  return join qq{\n}, @r;
@@ -226,7 +226,7 @@ See the C<new()> method.
 
 =head1 DEPENDENCIES
 
-This modules has to be used inside the Net::DRI framework and needs the following components:
+This module has to be used inside the Net::DRI framework and needs the following components:
 
 =over
 
@@ -275,7 +275,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+Copyright (c) 2009,2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
