@@ -1,6 +1,6 @@
 ## Domain Registry Interface, AFNIC WS Message
 ##
-## Copyright (c) 2005,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005,2008-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,13 +18,14 @@
 package Net::DRI::Protocol::AFNIC::WS::Message;
 
 use strict;
+use warnings;
 
 use Net::DRI::Protocol::ResultStatus;
 
 use base qw(Class::Accessor::Chained::Fast Net::DRI::Protocol::Message);
 __PACKAGE__->mk_accessors(qw(version service method params result errcode));
 
-our $VERSION=do { my @r=(q$Revision: 1.7 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.9 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -54,7 +55,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005,2008-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -85,7 +86,7 @@ sub new
  return $self;
 }
 
-sub as_string 
+sub as_string
 {
  my ($self)=@_;
  my @p=@{$self->params()};
@@ -112,7 +113,7 @@ sub parse
 }
 
 ## We handle all non free cases as errors, even if we should not
-sub is_success 
+sub is_success
 {
  my $self=shift;
  my $r=$self->result();
@@ -127,8 +128,8 @@ sub result_status
  my $self=shift;
  my $r=$self->result();
 
- return Net::DRI::Protocol::ResultStatus->new_success('COMMAND_SUCCESSFUL',$r->{message}) if ($r->{free});
- 
+ return Net::DRI::Protocol::ResultStatus->new_success($r->{message}) if $r->{free};
+
  my %codes=( 0   => 2400, # problme de connexion  la base de donnes => Command failed
              1   => 2302, # le nom de domaine est dj enregistr => Object exists
              2   => 2308, # un nom de domaine est dj enregistr  l'identique dans l'une des extensions du domaine public => Data management policy violation
@@ -139,8 +140,7 @@ sub result_status
            );
 
  my $code=$self->errcode();
- my $eppcode=(!defined($code) || $code >=1000 || !exists($codes{$code}))? 'GENERIC_ERROR' : $codes{$code};
-
+ my $eppcode=(defined $code && exists $codes{$code})? $codes{$code} : 'COMMAND_FAILED';
  return Net::DRI::Protocol::ResultStatus->new('afnic_ws_check_domain',$code,$eppcode,$self->is_success(),$r->{message});
  ## Warning: when we handle multiple web services, we will need a way to retrieve the method name called,
  ## to find the correct key of the hash (and special case of free <=> 2303)
