@@ -1,6 +1,6 @@
 ## Domain Registry Interface, BookMyName Web Services Message
 ##
-## Copyright (c) 2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2008-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,13 +18,14 @@
 package Net::DRI::Protocol::BookMyName::WS::Message;
 
 use strict;
+use warnings;
 
 use Net::DRI::Protocol::ResultStatus;
 
 use base qw(Class::Accessor::Chained::Fast Net::DRI::Protocol::Message);
 __PACKAGE__->mk_accessors(qw(version method params operation result retcode retval));
 
-our $VERSION=do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.6 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -54,7 +55,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2008-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -114,6 +115,12 @@ our %CODES=(	domain_info => {	'-1,-1' => 2200,
 					'-1,-3' => 2003,
 					'0,0' => 2303,
 				},
+		domain_check => {	'-1,-1' => 2200,
+					'-1,-2' => 2303,
+					'-1,-3' => 2103,
+					'-1,-4' => 2303,
+					'-1,-5' => 2003,
+				},
 		account_list_domains =>	{
 					'-1,-1' => 2200,
 					},
@@ -126,10 +133,9 @@ sub result_status
  my $self=shift;
  my ($op,$rc,$rv)=($self->operation(),$self->retcode(),$self->retval());
  my $ok=$self->is_success();
-
  my $k=$rc.','.$rv;
- my $eppcode=(exists($CODES{$op}) && (ref($CODES{$op}) eq 'HASH') && keys(%{$CODES{$op}}) && exists($CODES{$op}->{$k}))? $CODES{$op}->{$k}  : 'GENERIC_ERROR';
- return Net::DRI::Protocol::ResultStatus->new('bookmyname_ws',100*$rc+$rv,$ok? 'COMMAND_SUCCESSFUL' : $eppcode,$ok,'retcode='.$rc.' retval='.$rv,'en');
+ my $eppcode=(exists $CODES{$op} && ref $CODES{$op} eq 'HASH' && keys(%{$CODES{$op}}) && exists $CODES{$op}->{$k})? $CODES{$op}->{$k} : 'COMMAND_FAILED';
+ return Net::DRI::Protocol::ResultStatus->new('bookmyname_ws',100*$rc+$rv,$ok ? 'COMMAND_SUCCESSFUL' : $eppcode,$ok,'retcode='.$rc.' retval='.$rv,'en');
 }
 
 ####################################################################################################
