@@ -1,6 +1,6 @@
-## Domain Registry Interface, EPP Protocol (RFC 4930,4931,4932,4933,4934,3735)
+## Domain Registry Interface, EPP Protocol (STD 69)
 ##
-## Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,22 +18,21 @@
 package Net::DRI::Protocol::EPP;
 
 use strict;
+use warnings;
 
 use base qw(Net::DRI::Protocol);
 
 use Net::DRI::Util;
-
 use Net::DRI::Protocol::EPP::Message;
 use Net::DRI::Protocol::EPP::Core::Status;
-use Net::DRI::Data::Contact;
 
-our $VERSION=do { my @r=(q$Revision: 1.12 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.15 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
 =head1 NAME
 
-Net::DRI::Protocol::EPP - EPP Protocol (RFC 4930,4931,4932,4933,4934 obsoleting RFC 3730,3731,3732,3733,3734 and RFC 3735) for Net::DRI
+Net::DRI::Protocol::EPP - EPP Protocol (STD 69 aka RFC 5730,5731,5732,5733,5734 and RFC 3735) for Net::DRI
 
 =head1 DESCRIPTION
 
@@ -57,7 +56,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -73,12 +72,11 @@ See the LICENSE file that comes with this distribution for more details.
 
 sub new
 {
- my $c=shift;
- my ($drd,$version,$extrah,$coremods)=@_;
-
- my $self=$c->SUPER::new();
+ my ($c,$ctx,$rp)=@_;
+ my $drd=$ctx->{registry}->driver();
+ my $self=$c->SUPER::new($ctx);
  $self->name('EPP');
- $version=Net::DRI::Util::check_equal($version,['1.0'],'1.0');
+ my $version=Net::DRI::Util::check_equal($rp->{version},['1.0'],'1.0');
  $self->version($version);
 
  foreach my $o (qw/ip status/) { $self->capabilities('host_update',$o,['add','del']); }
@@ -94,72 +92,55 @@ sub new
  $self->{usenullauth}=$drd->info('use_null_auth') || 0; ## See RFC4931 3.2.5
  $self->ns({ _main   => ['urn:ietf:params:xml:ns:epp-1.0','epp-1.0.xsd'],
              domain  => ['urn:ietf:params:xml:ns:domain-1.0','domain-1.0.xsd'],
-             host    => ['urn:ietf:params:xml:ns:host-1.0','host-1.0.xsd'],
              contact => ['urn:ietf:params:xml:ns:contact-1.0','contact-1.0.xsd'],
            });
 
+ $drd->set_factories($self) if $drd->can('set_factories');
  $self->factories('message',sub { my $m=Net::DRI::Protocol::EPP::Message->new(@_); $m->ns($self->ns()); $m->version($version); return $m; });
  $self->factories('status',sub { return Net::DRI::Protocol::EPP::Core::Status->new(); });
- $self->factories('contact',sub { return Net::DRI::Data::Contact->new(); });
 
- $self->_load($extrah,$coremods);
+ $self->_load($rp);
+ $self->setup($rp);
  return $self;
 }
 
 sub _load
 {
- my ($self,$extrah,$coremods)=@_;
- my (@core,@class);
-
- if (defined($coremods))
- {
-  @core=(ref($coremods) eq 'ARRAY')? @$coremods : ($coremods);
- } else
- {
-  @core=qw/Session RegistryMessage Domain Contact/;
-  push @core,'Host' unless $self->{hostasattr};
- }
- push @class,map { /::/? $_ : 'Net::DRI::Protocol::EPP::Core::'.$_ } @core;
- if (defined($extrah) && $extrah)
- {
-  push @class,map { my $f=$_; $f=~s!/!::!g; $f; } map { /::/? $_ : 'Net::DRI::Protocol::EPP::Extensions::'.$_ } (ref($extrah)? @$extrah : ($extrah));
- }
-
+ my ($self,$rp)=@_;
+ my $extramods=$rp->{extensions};
+ my @class=$self->core_modules($rp);
+ push @class,map { 'Net::DRI::Protocol::EPP::Extensions::'.$_; } $self->default_extensions($rp) if $self->can('default_extensions');
+ push @class,map { my $f=$_; $f='Net::DRI::Protocol::EPP::Extensions::'.$f unless ($f=~s/^\+//); $f; } (ref $extramods ? @$extramods : ($extramods)) if defined $extramods && $extramods;
  $self->SUPER::_load(@class);
 }
 
-sub server_greeting { my ($self,$v)=@_; $self->{server_greeting}=$v if $v; return $self->{server_greeting}; }
+sub setup {} ## subclass as needed
 
-sub parse_status
+sub core_modules
 {
- my $node=shift;
- my %tmp;
- $tmp{name}=$node->getAttribute('s');
- $tmp{lang}=$node->getAttribute('lang') || 'en';
- $tmp{msg}=$node->firstChild()->getData() if ($node->firstChild());
- return \%tmp;
+ my ($self,$rp)=@_;
+ my @core=qw/Session RegistryMessage Domain Contact/;
+ if (! $self->{hostasattr})
+ {
+  push @core,'Host';
+  $self->ns({host => ['urn:ietf:params:xml:ns:host-1.0','host-1.0.xsd']});
+ }
+ return map { 'Net::DRI::Protocol::EPP::Core::'.$_ } @core;
 }
 
-sub core_contact_types { return ('admin','tech','billing'); }
+sub core_contact_types { return qw/admin tech billing/; }
 
 sub ns
 {
  my ($self,$add)=@_;
- $self->{ns}={ ref($self->{ns})? %{$self->{ns}} : (), %$add } if (defined($add) && ref($add) eq 'HASH');
+ $self->{ns}={ ref($self->{ns})? %{$self->{ns}} : (), %$add } if defined $add && ref $add eq 'HASH';
  return $self->{ns};
 }
 
-## TODO : should that be in EPP/Connection ? <=> see IRIS/XCP.pm
-## Was previously %PROTOCOL_DEFAULT_EPP in DRD.pm
 sub transport_default
 {
- my ($self,$name)=@_;
- my %r;
- if (defined $name && $name eq 'socket_inet')
- {
-  %r=(defer => 0, socktype => 'ssl', ssl_cipher_list => 'TLSv1', remote_port => 700, protocol_connection => 'Net::DRI::Protocol::EPP::Connection', protocol_version => 1);
- }
- return \%r;
+ my ($self)=@_;
+ return (protocol_connection => 'Net::DRI::Protocol::EPP::Connection', protocol_version => 1);
 }
 
 ####################################################################################################
