@@ -1,6 +1,6 @@
-## Domain Registry Interface, EPP Contact commands (RFC4933)
+## Domain Registry Interface, EPP Contact commands (RFC5733)
 ##
-## Copyright (c) 2005,2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,20 +18,19 @@
 package Net::DRI::Protocol::EPP::Core::Contact;
 
 use strict;
+use warnings;
 
 use Net::DRI::Util;
 use Net::DRI::Exception;
-use Net::DRI::Protocol::EPP;
+use Net::DRI::Protocol::EPP::Util;
 
-use DateTime::Format::ISO8601;
-
-our $VERSION=do { my @r=(q$Revision: 1.16 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.19 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
 =head1 NAME
 
-Net::DRI::Protocol::EPP::Core::Contact - EPP Contact commands (RFC4933 obsoleting RFC3733) for Net::DRI
+Net::DRI::Protocol::EPP::Core::Contact - EPP Contact commands (RFC5733 obsoleting RFC4933 and RFC3733) for Net::DRI
 
 =head1 DESCRIPTION
 
@@ -55,7 +54,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -109,13 +108,9 @@ sub build_command
 
  if (($tcommand=~m/^(?:info|transfer)$/) && ref($contact[0]) && Net::DRI::Util::isa_contact($contact[0]))
  {
-  my $az=$contact[0]->auth();
-  if ($az && ref($az) && exists($az->{pw}))
-  {
-   push @d,['contact:authInfo',['contact:pw',$az->{pw}]];
-  }
+  push @d,build_authinfo($contact[0]);
  }
- 
+
  return @d;
 }
 
@@ -137,26 +132,24 @@ sub check_parse
  return unless $mes->is_success();
 
  my $chkdata=$mes->get_response('contact','chkData');
- return unless $chkdata;
+ return unless defined $chkdata;
  foreach my $cd ($chkdata->getChildrenByTagNameNS($mes->ns('contact'),'cd'))
  {
-  my $c=$cd->getFirstChild();
   my $contact;
-  while($c)
+  foreach my $el (Net::DRI::Util::xml_list_children($cd))
   {
-   next unless ($c->nodeType() == 1); ## only for element nodes
-   my $n=$c->localname() || $c->nodeName();
+   my ($n,$c)=@$el;
    if ($n eq 'id')
    {
-    $contact=$c->getFirstChild()->getData();
+    $contact=$c->textContent();
     $rinfo->{contact}->{$contact}->{action}='check';
     $rinfo->{contact}->{$contact}->{exist}=1-Net::DRI::Util::xml_parse_boolean($c->getAttribute('avail'));
    }
    if ($n eq 'reason')
    {
-    $rinfo->{contact}->{$contact}->{exist_reason}=$c->getFirstChild()->getData();
+    $rinfo->{contact}->{$contact}->{exist_reason}=$c->textContent();
    }
-  } continue { $c=$c->getNextSibling(); }
+  }
  }
 }
 
@@ -175,59 +168,56 @@ sub info_parse
  return unless $mes->is_success();
 
  my $infdata=$mes->get_response('contact','infData');
- return unless $infdata;
+ return unless defined $infdata;
 
- my %cd=map { $_ => [] } qw/name org street city sp pc cc/;
- my $contact=$po->factories()->{contact}->();
+ my %cd=map { $_ => [] } qw/name org city sp pc cc/;
+ $cd{street}=[[],[]];
+ my $contact=$po->create_local_object('contact');
  my @s;
- my $pd=DateTime::Format::ISO8601->new();
- my $c=$infdata->getFirstChild();
- while ($c)
+
+ foreach my $el (Net::DRI::Util::xml_list_children($infdata))
  {
-  next unless ($c->nodeType() == 1);
-  my $name=$c->localname() || $c->nodeName();
-  next unless $name;
+  my ($name,$c)=@$el;
   if ($name eq 'id')
   {
-   $oname=$c->getFirstChild()->getData();
+   $oname=$c->textContent();
    $rinfo->{contact}->{$oname}->{action}='info';
    $rinfo->{contact}->{$oname}->{exist}=1;
    $rinfo->{contact}->{$oname}->{id}=$oname;
    $contact->srid($oname);
   } elsif ($name eq 'roid')
   {
-   $contact->roid($c->getFirstChild()->getData());
+   $contact->roid($c->textContent());
    $rinfo->{contact}->{$oname}->{roid}=$contact->roid();
   } elsif ($name eq 'status')
   {
-   push @s,Net::DRI::Protocol::EPP::parse_status($c);
+   push @s,Net::DRI::Protocol::EPP::Util::parse_node_status($c);
   } elsif ($name=~m/^(clID|crID|upID)$/)
   {
-   $rinfo->{contact}->{$oname}->{$1}=get_data($c);
+   $rinfo->{contact}->{$oname}->{$1}=$c->textContent();
   } elsif ($name=~m/^(crDate|upDate|trDate)$/)
   {
-   $rinfo->{contact}->{$oname}->{$1}=$pd->parse_datetime($c->getFirstChild()->getData());
+   $rinfo->{contact}->{$oname}->{$1}=$po->parse_iso8601($c->textContent());
   } elsif ($name eq 'email')
   {
-   $contact->email($c->getFirstChild()->getData());
+   $contact->email($c->textContent());
   } elsif ($name eq 'voice')
   {
-   $contact->voice(parse_tel($c));
+   $contact->voice(Net::DRI::Protocol::EPP::Util::parse_tel($c));
   } elsif ($name eq 'fax')
   {
-   $contact->fax(parse_tel($c));
+   $contact->fax(Net::DRI::Protocol::EPP::Util::parse_tel($c));
   } elsif ($name eq 'postalInfo')
   {
    parse_postalinfo($po,$c,\%cd);
   } elsif ($name eq 'authInfo') ## we only try to parse the authInfo version defined in the RFC, other cases are to be handled by extensions
   {
-   my $n=$c->getChildrenByTagNameNS($mes->ns('contact'),'pw');
-   $contact->auth({pw => $n->size()? $n->shift()->getFirstChild()->getData() : undef});
+   $contact->auth({pw => scalar Net::DRI::Util::xml_child_content($c,$mes->ns('contact'),'pw')});
   } elsif ($name eq 'disclose')
   {
    $contact->disclose(parse_disclose($c));
   }
- } continue { $c=$c->getNextSibling(); }
+ }
 
  $contact->name(@{$cd{name}});
  $contact->org(@{$cd{org}});
@@ -241,21 +231,6 @@ sub info_parse
  $rinfo->{contact}->{$oname}->{self}=$contact;
 }
 
-sub parse_tel
-{
- my $node=shift;
- my $ext=$node->getAttribute('x') || '';
- my $num=get_data($node);
- $num.='x'.$ext if $ext;
- return $num;
-}
-
-sub get_data
-{
- my $n=shift;
- return ($n->getFirstChild())? $n->getFirstChild()->getData() : '';
-}
-
 sub parse_postalinfo
 {
  my ($epp,$c,$rcd)=@_;
@@ -263,47 +238,41 @@ sub parse_postalinfo
  $type=$epp->{defaulti18ntype} if (!defined($type) && defined($epp->{defaulti18ntype}));
  my $ti={loc=>0,int=>1}->{$type};
 
- my $n=$c->getFirstChild();
- while($n)
+ foreach my $el (Net::DRI::Util::xml_list_children($c))
  {
-  next unless ($n->nodeType() == 1);
-  my $name=$n->localname() || $n->nodeName();
-  next unless $name;
+  my ($name,$n)=@$el;
   if ($name eq 'name')
   {
-   $rcd->{name}->[$ti]=get_data($n);
+   $rcd->{name}->[$ti]=$n->textContent();
   } elsif ($name eq 'org')
   {
-   $rcd->{org}->[$ti]=get_data($n);
+   $rcd->{org}->[$ti]=$n->textContent();
   } elsif ($name eq 'addr')
   {
-   my $nn=$n->getFirstChild();
    my @street;
-   while($nn)
+   foreach my $sel (Net::DRI::Util::xml_list_children($n))
    {
-    next unless ($nn->nodeType() == 1);
-    my $name2=$nn->localname() || $nn->nodeName();
-    next unless $name2;
+    my ($name2,$nn)=@$sel;
     if ($name2 eq 'street')
     {
-     push @street,get_data($nn);
+     push @street,$nn->textContent();
     } elsif ($name2 eq 'city')
     {
-     $rcd->{city}->[$ti]=get_data($nn);
+     $rcd->{city}->[$ti]=$nn->textContent();
     } elsif ($name2 eq 'sp')
     {
-     $rcd->{sp}->[$ti]=get_data($nn);
+     $rcd->{sp}->[$ti]=$nn->textContent();
     } elsif ($name2 eq 'pc')
     {
-     $rcd->{pc}->[$ti]=get_data($nn);
+     $rcd->{pc}->[$ti]=$nn->textContent();
     } elsif ($name2 eq 'cc')
     {
-     $rcd->{cc}->[$ti]=get_data($nn);
+     $rcd->{cc}->[$ti]=$nn->textContent();
     }
-   } continue { $nn=$nn->getNextSibling(); }
+   }
    $rcd->{street}->[$ti]=\@street;
   }
- } continue { $n=$n->getNextSibling(); }
+ }
 }
 
 sub parse_disclose ## RFC 4933 2.9
@@ -311,12 +280,9 @@ sub parse_disclose ## RFC 4933 2.9
  my $c=shift;
  my $flag=Net::DRI::Util::xml_parse_boolean($c->getAttribute('flag'));
  my %tmp;
- my $n=$c->getFirstChild();
- while($n)
+ foreach my $el (Net::DRI::Util::xml_list_children($c))
  {
-  next unless ($n->nodeType() == 1);
-  my $name=$n->localname() || $n->nodeName();
-  next unless $name;
+  my ($name,$n)=@$el;
   if ($name=~m/^(name|org|addr)$/)
   {
    my $t=$n->getAttribute('type');
@@ -325,7 +291,7 @@ sub parse_disclose ## RFC 4933 2.9
   {
    $tmp{$1}=$flag;
   }
- } continue { $n=$n->getNextSibling(); }
+ }
  return \%tmp;
 }
 
@@ -344,46 +310,29 @@ sub transfer_parse
  return unless $mes->is_success();
 
  my $trndata=$mes->get_response('contact','trnData');
- return unless $trndata;
+ return unless defined $trndata;
 
- my $pd=DateTime::Format::ISO8601->new();
- my $c=$trndata->getFirstChild();
- while ($c)
+ foreach my $el (Net::DRI::Util::xml_list_children($trndata))
  {
-  next unless ($c->nodeType() == 1); ## only for element nodes
-  my $name=$c->localname() || $c->nodeName();
-  next unless $name;
-
+  my ($name,$c)=@$el;
   if ($name eq 'id')
   {
-   $oname=$c->getFirstChild()->getData();
+   $oname=$c->textContent();
    $rinfo->{contact}->{$oname}->{id}=$oname;
    $rinfo->{contact}->{$oname}->{action}='transfer';
    $rinfo->{contact}->{$oname}->{exist}=1;
   } elsif ($name=~m/^(trStatus|reID|acID)$/)
   {
-   $rinfo->{contact}->{$oname}->{$1}=$c->getFirstChild()->getData();
+   $rinfo->{contact}->{$oname}->{$1}=$c->textContent();
   } elsif ($name=~m/^(reDate|acDate)$/)
   {
-   $rinfo->{contact}->{$oname}->{$1}=$pd->parse_datetime($c->getFirstChild()->getData());
+   $rinfo->{contact}->{$oname}->{$1}=$po->parse_iso8601($c->textContent());
   }
- } continue { $c=$c->getNextSibling(); }
+ }
 }
 
 ############ Transform commands
 
-sub build_tel
-{
- my ($name,$tel)=@_;
- if ($tel=~m/^(\S+)x(\S+)$/)
- {
-  return [$name,$1,{x=>$2}];
- } else
- {
-  return [$name,$tel];
- }
-}
-
 sub build_authinfo
 {
  my $contact=shift;
@@ -445,8 +394,8 @@ sub build_cdata
  push @d,['contact:postalInfo',@postl,{type=>'loc'}] if (($v & 5) && $hasloc); ## loc+int OR loc
  push @d,['contact:postalInfo',@posti,{type=>'int'}] if (($v & 6) && $hasint); ## loc+int OR int
 
- push @d,build_tel('contact:voice',$contact->voice()) if defined($contact->voice());
- push @d,build_tel('contact:fax',$contact->fax()) if defined($contact->fax());
+ push @d,Net::DRI::Protocol::EPP::Util::build_tel('contact:voice',$contact->voice()) if defined($contact->voice());
+ push @d,Net::DRI::Protocol::EPP::Util::build_tel('contact:fax',$contact->fax()) if defined($contact->fax());
  push @d,['contact:email',$contact->email()] if defined($contact->email());
  push @d,build_authinfo($contact);
  push @d,build_disclose($contact);
@@ -489,26 +438,24 @@ sub create_parse
  return unless $mes->is_success();
 
  my $credata=$mes->get_response('contact','creData');
- return unless $credata;
+ return unless defined $credata;
 
- my $c=$credata->getFirstChild();
- while ($c)
+ foreach my $el (Net::DRI::Util::xml_list_children($credata))
  {
-  next unless ($c->nodeType() == 1); ## only for element nodes
-  my $name=$c->localname() || $c->nodeName();
+  my ($name,$c)=@$el;
   if ($name eq 'id')
   {
-   my $new=$c->getFirstChild()->getData();
-   $rinfo->{contact}->{$oname}->{id}=$new if (defined($oname) && ($oname ne $new)); ## registry may give another id than the one we requested or not take ours into account at all !
+   my $new=$c->textContent();
+   $rinfo->{contact}->{$oname}->{id}=$new if (defined $oname && ($oname ne $new)); ## registry may give another id than the one we requested or not take ours into account at all !
    $oname=$new;
    $rinfo->{contact}->{$oname}->{id}=$oname;
    $rinfo->{contact}->{$oname}->{action}='create';
    $rinfo->{contact}->{$oname}->{exist}=1;
   } elsif ($name=~m/^(crDate)$/)
   {
-   $rinfo->{contact}->{$oname}->{$1}=DateTime::Format::ISO8601->new()->parse_datetime($c->getFirstChild()->getData());
+   $rinfo->{contact}->{$oname}->{$1}=$po->parse_iso8601($c->textContent());
   }
- } continue { $c=$c->getNextSibling(); }
+ }
 }
 
 sub delete
@@ -537,9 +484,9 @@ sub transfer_cancel
 
 sub transfer_answer
 {
- my ($epp,$c,$approve)=@_;
+ my ($epp,$c,$ep)=@_;
  my $mes=$epp->message();
- my @d=build_command($mes,['transfer',{'op'=>((defined($approve) && $approve)? 'approve' : 'reject' )}],$c);
+ my @d=build_command($mes,['transfer',{'op'=>((Net::DRI::Util::has_key($ep,'approve') && $ep->{approve})? 'approve' : 'reject' )}],$c);
  $mes->command_body(\@d);
 }
 
@@ -548,23 +495,17 @@ sub update
  my ($epp,$contact,$todo)=@_;
  my $mes=$epp->message();
 
- Net::DRI::Exception::usererr_invalid_parameters($todo.' must be a Net::DRI::Data::Changes object') unless Net::DRI::Util::isa_changes($todo);
- if ((grep { ! /^(?:add|del)$/ } $todo->types('status')) ||
-     (grep { ! /^(?:set)$/ } $todo->types('info'))
-    )
- {
-  Net::DRI::Exception->die(0,'protocol/EPP',11,'Only status add/del or info set available for contact');
- }
-
- my @d=build_command($mes,'update',$contact);
+ Net::DRI::Exception::usererr_invalid_parameters($todo.' must be a non empty Net::DRI::Data::Changes object') unless Net::DRI::Util::isa_changes($todo);
 
  my $sadd=$todo->add('status');
  my $sdel=$todo->del('status');
- push @d,['contact:add',$sadd->build_xml('contact:status')] if ($sadd);
- push @d,['contact:rem',$sdel->build_xml('contact:status')] if ($sdel);
+
+ my @d=build_command($mes,'update',$contact);
+ push @d,['contact:add',$sadd->build_xml('contact:status')] if Net::DRI::Util::isa_statuslist($sadd);
+ push @d,['contact:rem',$sdel->build_xml('contact:status')] if Net::DRI::Util::isa_statuslist($sdel);
 
  my $newc=$todo->set('info');
- if ($newc)
+ if (defined $newc)
  {
   Net::DRI::Exception->die(1,'protocol/EPP',10,'Invalid contact '.$newc) unless Net::DRI::Util::isa_contact($newc);
   $newc->validate(1); ## will trigger an Exception if needed
@@ -584,30 +525,27 @@ sub pandata_parse
  return unless $mes->is_success();
 
  my $pandata=$mes->get_response('contact','panData');
- return unless $pandata;
+ return unless defined $pandata;
 
- my $c=$pandata->firstChild();
- while ($c)
+ foreach my $el (Net::DRI::Util::xml_list_children($pandata))
  {
-  next unless ($c->nodeType() == 1); ## only for element nodes
-  my $name=$c->localname() || $c->nodeName();
-  next unless $name;
-
+  my ($name,$c)=@$el;
   if ($name eq 'id')
   {
-   $oname=$c->getFirstChild()->getData();
+   $oname=$c->textContent();
    $rinfo->{contact}->{$oname}->{action}='review';
    $rinfo->{contact}->{$oname}->{result}=Net::DRI::Util::xml_parse_boolean($c->getAttribute('paResult'));
   } elsif ($name eq 'paTRID')
   {
-   my @tmp=$c->getChildrenByTagNameNS($mes->ns('_main'),'clTRID');
-   $rinfo->{contact}->{$oname}->{trid}=$tmp[0]->getFirstChild()->getData() if (@tmp && $tmp[0]);
-   $rinfo->{contact}->{$oname}->{svtrid}=($c->getChildrenByTagNameNS($mes->ns('_main'),'svTRID'))[0]->getFirstChild()->getData();
+   my $ns=$mes->ns('_main');
+   my $tmp=Net::DRI::Util::xml_child_content($c,$ns,'clTRID');
+   $rinfo->{contact}->{$oname}->{trid}=$tmp if defined $tmp;
+   $rinfo->{contact}->{$oname}->{svtrid}=Net::DRI::Util::xml_child_content($c,$ns,'svTRID');
   } elsif ($name eq 'paDate')
   {
-   $rinfo->{contact}->{$oname}->{date}=DateTime::Format::ISO8601->new()->parse_datetime($c->firstChild->getData());
+   $rinfo->{contact}->{$oname}->{date}=$po->parse_iso8601($c->textContent());
   }
- } continue { $c=$c->getNextSibling(); }
+ }
 }
 
 ####################################################################################################
