@@ -1,6 +1,6 @@
-## Domain Registry Interface, EPP Host commands (RFC4932)
+## Domain Registry Interface, EPP Host commands (RFC5732)
 ##
-## Copyright (c) 2005,2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,21 +18,19 @@
 package Net::DRI::Protocol::EPP::Core::Host;
 
 use strict;
+use warnings;
 
 use Net::DRI::Util;
 use Net::DRI::Exception;
-use Net::DRI::Data::Hosts;
-use Net::DRI::Protocol::EPP;
+use Net::DRI::Protocol::EPP::Util;
 
-use DateTime::Format::ISO8601;
-
-our $VERSION=do { my @r=(q$Revision: 1.13 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.16 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
 =head1 NAME
 
-Net::DRI::Protocol::EPP::Core::Host - EPP Host commands (RFC4932 obsoleting RFC3732) for Net::DRI
+Net::DRI::Protocol::EPP::Core::Host - EPP Host commands (RFC5732 obsoleting RFC4932 and RFC3732) for Net::DRI
 
 =head1 DESCRIPTION
 
@@ -56,7 +54,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -114,7 +112,6 @@ sub check
  $mes->command_body(\@d);
 }
 
-
 sub check_parse
 {
  my ($po,$otype,$oaction,$oname,$rinfo)=@_;
@@ -122,26 +119,24 @@ sub check_parse
  return unless $mes->is_success();
 
  my $chkdata=$mes->get_response('host','chkData');
- return unless $chkdata;
+ return unless defined $chkdata;
  foreach my $cd ($chkdata->getChildrenByTagNameNS($mes->ns('host'),'cd'))
  {
-  my $c=$cd->getFirstChild();
   my $host;
-  while($c)
+  foreach my $el (Net::DRI::Util::xml_list_children($cd))
   {
-   next unless ($c->nodeType() == 1); ## only for element nodes
-   my $n=$c->localname() || $c->nodeName();
+   my ($n,$c)=@$el;
    if ($n eq 'name')
    {
-    $host=lc($c->getFirstChild()->getData());
+    $host=lc($c->textContent());
     $rinfo->{host}->{$host}->{action}='check';
     $rinfo->{host}->{$host}->{exist}=1-Net::DRI::Util::xml_parse_boolean($c->getAttribute('avail'));
    }
    if ($n eq 'reason')
    {
-    $rinfo->{host}->{$host}->{exist_reason}=$c->getFirstChild()->getData();
+    $rinfo->{host}->{$host}->{exist_reason}=$c->textContent();
    }
-  } continue { $c=$c->getNextSibling(); }
+  }
  }
 }
 
@@ -160,46 +155,41 @@ sub info_parse
  return unless $mes->is_success();
 
  my $infdata=$mes->get_response('host','infData');
- return unless $infdata;
+ return unless defined $infdata;
 
  my (@s,@ip4,@ip6);
- my $pd=DateTime::Format::ISO8601->new();
- my $c=$infdata->getFirstChild();
- while ($c)
+ foreach my $el (Net::DRI::Util::xml_list_children($infdata))
  {
-  next unless ($c->nodeType() == 1); ## only for element nodes
-  my $name=$c->localname() || $c->nodeName();
-  next unless $name;
-
+  my ($name,$c)=@$el;
   if ($name eq 'name')
   {
-   $oname=lc($c->getFirstChild()->getData());
+   $oname=lc($c->textContent());
    $rinfo->{host}->{$oname}->{action}='info';
    $rinfo->{host}->{$oname}->{exist}=1;
   } elsif ($name=~m/^(clID|crID|upID)$/)
   {
-   $rinfo->{host}->{$oname}->{$1}=$c->getFirstChild()->getData();
+   $rinfo->{host}->{$oname}->{$1}=$c->textContent();
   } elsif ($name=~m/^(crDate|upDate|trDate)$/)
   {
-   $rinfo->{host}->{$oname}->{$1}=$pd->parse_datetime($c->getFirstChild()->getData());
+   $rinfo->{host}->{$oname}->{$1}=$po->parse_iso8601($c->textContent());
   } elsif ($name eq 'roid')
   {
-   $rinfo->{host}->{$oname}->{roid}=$c->getFirstChild()->getData();
+   $rinfo->{host}->{$oname}->{roid}=$c->textContent();
   } elsif ($name eq 'status')
   {
-   push @s,Net::DRI::Protocol::EPP::parse_status($c);
+   push @s,Net::DRI::Protocol::EPP::Util::parse_node_status($c);
   } elsif ($name eq 'addr')
   {
-   my $ip=$c->getFirstChild()->getData();
+   my $ip=$c->textContent();
    my $ipv=$c->getAttribute('ip');
    $ipv='v4' unless (defined($ipv) && $ipv);
    push @ip4,$ip if ($ipv eq 'v4');
    push @ip6,$ip if ($ipv eq 'v6');
   }
- } continue { $c=$c->getNextSibling(); }
+ }
 
  $rinfo->{host}->{$oname}->{status}=$po->create_local_object('status')->add(@s);
- $rinfo->{host}->{$oname}->{self}=Net::DRI::Data::Hosts->new($oname,\@ip4,\@ip6,1);
+ $rinfo->{host}->{$oname}->{self}=$po->create_local_object('hosts',$oname,\@ip4,\@ip6,1);
 }
 
 ############ Transform commands
@@ -209,7 +199,7 @@ sub create
  my ($epp,$ns)=@_;
  my $mes=$epp->message();
  my @d=build_command($mes,'create',$ns);
- push @d,add_ip($ns);
+ push @d,add_ip($ns) if Net::DRI::Util::isa_hosts($ns);
  $mes->command_body(\@d);
 }
 
@@ -219,26 +209,22 @@ sub create_parse
  my $mes=$po->message();
  return unless $mes->is_success();
 
- my $infdata=$mes->get_response('host','creData');
- return unless $infdata;
+ my $credata=$mes->get_response('host','creData');
+ return unless defined $credata;
 
- my $c=$infdata->getFirstChild();
- while ($c) ## host:name is not used
+ foreach my $el (Net::DRI::Util::xml_list_children($credata))
  {
-  next unless ($c->nodeType() == 1); ## only for element nodes
-  my $name=$c->localname() || $c->nodeName();
-  next unless $name;
- 
+  my ($name,$c)=@$el;
   if ($name eq 'name')
   {
-   $oname=lc($c->getFirstChild()->getData());
+   $oname=lc($c->textContent());
    $rinfo->{host}->{$oname}->{action}='create';
    $rinfo->{host}->{$oname}->{exist}=1;
   } elsif ($name=~m/^(crDate)$/)
   {
-   $rinfo->{host}->{$oname}->{$1}=DateTime::Format::ISO8601->new()->parse_datetime($c->getFirstChild()->getData());
+   $rinfo->{host}->{$oname}->{$1}=$po->parse_iso8601($c->textContent());
   }
- } continue { $c=$c->getNextSibling(); }
+ }
 }
 
 sub delete
@@ -254,14 +240,7 @@ sub update
  my ($epp,$ns,$todo)=@_;
  my $mes=$epp->message();
 
- Net::DRI::Exception::usererr_invalid_parameters($todo.' must be a Net::DRI::Data::Changes object') unless Net::DRI::Util::isa_changes($todo);
- if ((grep { ! /^(?:add|del)$/ } $todo->types('ip')) ||
-     (grep { ! /^(?:add|del)$/ } $todo->types('status')) ||
-     (grep { ! /^(?:set)$/ } $todo->types('name'))
-    )
- {
-  Net::DRI::Exception->die(0,'protocol/EPP',11,'Only IP/status add/del or name set available for host');
- }
+ Net::DRI::Exception::usererr_invalid_parameters($todo.' must be a non empty Net::DRI::Data::Changes object') unless Net::DRI::Util::isa_changes($todo);
 
  my $nsadd=$todo->add('ip');
  my $nsdel=$todo->del('ip');
@@ -271,20 +250,21 @@ sub update
 
  unless (defined($ns) && $ns)
  {
-  $ns=$nsadd->get_names(1) if (defined($nsadd) && ref($nsadd) && UNIVERSAL::can($nsadd,'get_names'));
-  $ns=$nsdel->get_names(1) if (defined($nsdel) && ref($nsdel) && UNIVERSAL::can($nsdel,'get_names'));
+  $ns=$nsadd->get_names(1) if Net::DRI::Util::isa_hosts($nsadd);
+  $ns=$nsdel->get_names(1) if Net::DRI::Util::isa_hosts($nsdel);
  }
- my @d=build_command($mes,'update',$ns);
+
  my (@add,@rem);
- push @add,add_ip($nsadd)    if $nsadd;
- push @add,$sadd->build_xml('host:status') if $sadd;
- push @rem,add_ip($nsdel)    if $nsdel;
- push @rem,$sdel->build_xml('host:status') if $sdel;
+ push @add,add_ip($nsadd)                  if Net::DRI::Util::isa_hosts($nsadd);
+ push @add,$sadd->build_xml('host:status') if Net::DRI::Util::isa_statuslist($sadd);
+ push @rem,add_ip($nsdel)                  if Net::DRI::Util::isa_hosts($nsdel);
+ push @rem,$sdel->build_xml('host:status') if Net::DRI::Util::isa_statuslist($sdel);
 
- push @d,['host:add',@add] if (@add);
- push @d,['host:rem',@rem] if (@rem);
+ my @d=build_command($mes,'update',$ns);
+ push @d,['host:add',@add] if @add;
+ push @d,['host:rem',@rem] if @rem;
 
- if (defined($newname) && $newname)
+ if (defined($newname) && length $newname)
  {
   Net::DRI::Exception->die(1,'protocol/EPP',10,'Invalid host name: '.$newname) unless Net::DRI::Util::is_hostname($newname);
   push @d,['host:chg',['host:name',$newname]];
@@ -295,7 +275,6 @@ sub update
 sub add_ip
 {
  my ($ns)=@_;
- return () unless (defined($ns) && ref($ns));
  my @ip;
  my ($name,$r4,$r6)=$ns->get_details(1);
  push @ip,map { ['host:addr',$_,{ip=>'v4'}] } @$r4 if @$r4;
@@ -313,30 +292,27 @@ sub pandata_parse
  return unless $mes->is_success();
 
  my $pandata=$mes->get_response('host','panData');
- return unless $pandata;
+ return unless defined $pandata;
 
- my $c=$pandata->firstChild();
- while ($c)
+ foreach my $el (Net::DRI::Util::xml_list_children($pandata))
  {
-  next unless ($c->nodeType() == 1); ## only for element nodes
-  my $name=$c->localname() || $c->nodeName();
-  next unless $name;
-
+  my ($name,$c)=@$el;
   if ($name eq 'name')
   {
-   $oname=lc($c->getFirstChild()->getData());
+   $oname=lc($c->textContent());
    $rinfo->{host}->{$oname}->{action}='review';
    $rinfo->{host}->{$oname}->{result}=Net::DRI::Util::xml_parse_boolean($c->getAttribute('paResult'));
   } elsif ($name eq 'paTRID')
   {
-   my @tmp=$c->getChildrenByTagNameNS($mes->ns('_main'),'clTRID');
-   $rinfo->{host}->{$oname}->{trid}=$tmp[0]->getFirstChild()->getData() if (@tmp && $tmp[0]);
-   $rinfo->{host}->{$oname}->{svtrid}=($c->getChildrenByTagNameNS($mes->ns('_main'),'svTRID'))[0]->getFirstChild()->getData();
+   my $ns=$mes->ns('_main');
+   my $tmp=Net::DRI::Util::xml_child_content($c,$ns,'clTRID');
+   $rinfo->{host}->{$oname}->{trid}=$tmp if defined $tmp;
+   $rinfo->{host}->{$oname}->{svtrid}=Net::DRI::Util::xml_child_content($c,$ns,'svTRID');
   } elsif ($name eq 'paDate')
   {
-   $rinfo->{host}->{$oname}->{date}=DateTime::Format::ISO8601->new()->parse_datetime($c->firstChild->getData());
+   $rinfo->{host}->{$oname}->{date}=$po->parse_iso8601($c->textContent());
   }
- } continue { $c=$c->getNextSibling(); }
+ }
 }
 
 ####################################################################################################
