@@ -1,7 +1,7 @@
 ## Domain Registry Interface, AFNIC (.FR/.RE) EPP extensions
 ## From http://www.afnic.fr/data/divers/public/afnic-epp-rc1.pdf (2008-06-30)
 ##
-## Copyright (c) 2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2008,2009,2011 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -25,7 +25,7 @@ use base qw/Net::DRI::Protocol::EPP/;
 use Net::DRI::Data::Contact::AFNIC;
 use Net::DRI::Protocol::EPP::Extensions::AFNIC::Status;
 
-our $VERSION=do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -55,7 +55,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2008,2009,2011 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -69,28 +69,21 @@ See the LICENSE file that comes with this distribution for more details.
 
 ####################################################################################################
 
-sub new
+sub setup
 {
- my ($c,$drd,$version,$extrah)=@_;
- my %e=map { $_ => 1 } (defined($extrah)? (ref($extrah)? @$extrah : ($extrah)) : ());
-
- $e{'Net::DRI::Protocol::EPP::Extensions::AFNIC::Domain'}=1;
- $e{'Net::DRI::Protocol::EPP::Extensions::AFNIC::Contact'}=1;
- $e{'Net::DRI::Protocol::EPP::Extensions::AFNIC::Notifications'}=1;
- $e{'Net::DRI::Protocol::EPP::Extensions::GracePeriod'}=1;
-
- my $self=$c->SUPER::new($drd,$version,[keys(%e)]);
- $self->ns({frnic=>['http://www.afnic.fr/xml/epp/frnic-1.0','frnic-1.0.xsd']}); ## fake xsd for now
+ my ($self,$rp)=@_;
+ $self->ns({frnic=>['http://www.afnic.fr/xml/epp/frnic-1.0','frnic-1.0.xsd']});
  $self->capabilities('domain_update','registrant',undef); ## a trade is required
  $self->capabilities('contact_update','status',undef); ## No changes in status possible for .FR contacts
  $self->capabilities('contact_update','disclose',['add','del']);
  $self->factories('contact',sub { return Net::DRI::Data::Contact::AFNIC->new(); });
  $self->factories('status',sub { return Net::DRI::Protocol::EPP::Extensions::AFNIC::Status->new(); });
  $self->default_parameters({domain_create => { ns => undef } }); ## No nameservers allowed during domain create
- return $self;
+ return;
 }
 
 sub core_contact_types { return ('admin','tech'); } ## No billing contact in .FR
+sub default_extensions { return qw/AFNIC::Domain AFNIC::Contact AFNIC::Notifications GracePeriod SecDNS/; }
 
 ####################################################################################################
 1;
