@@ -1,7 +1,7 @@
 ## Domain Registry Interface, EPP NSgroup extension commands
 ## (based on .BE Registration_guidelines_v4_7_1)
 ##
-## Copyright (c) 2005,2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -19,12 +19,12 @@
 package Net::DRI::Protocol::EPP::Extensions::NSgroup;
 
 use strict;
+use warnings;
 
 use Net::DRI::Util;
 use Net::DRI::Exception;
-use Net::DRI::Data::Hosts;
 
-our $VERSION=do { my @r=(q$Revision: 1.7 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.9 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -54,7 +54,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -89,7 +89,7 @@ sub ns
 {
  my ($mes)=@_;
  my $ns=$mes->ns('nsgroup');
- return defined($ns)? $ns : 'http://www.dns.be/xml/epp/nsgroup-1.0';
+ return defined $ns? $ns : 'http://www.dns.be/xml/epp/nsgroup-1.0';
 }
 
 sub build_command
@@ -97,31 +97,27 @@ sub build_command
  my ($epp,$msg,$command,$hosts)=@_;
 
  my @gn;
- foreach my $h ( grep { defined } (ref($hosts) eq 'ARRAY')? @$hosts : ($hosts))
+ foreach my $h ( grep { defined } (ref $hosts eq 'ARRAY')? @$hosts : ($hosts))
  {
   my $gn=Net::DRI::Util::isa_nsgroup($h)? $h->name() : $h;
-  Net::DRI::Exception->die(1,'protocol/EPP',10,'Invalid NSgroup name: '.$gn) unless (defined($gn) && $gn && !ref($gn) && Net::DRI::Util::xml_is_normalizedstring($gn,1,100));
+  Net::DRI::Exception->die(1,'protocol/EPP',10,'Invalid NSgroup name: '.$gn) unless (defined $gn && $gn && ! ref $gn && Net::DRI::Util::xml_is_normalizedstring($gn,1,100));
   push @gn,$gn;
  }
 
  Net::DRI::Exception->die(1,'protocol/EPP',2,'NSgroup name needed') unless @gn;
-
- my @ns=$msg->nsattrs('nsgroup');
- @ns=qw(http://www.dns.be/xml/epp/nsgroup-1.0 http://www.dns.be/xml/epp/nsgroup-1.0 nsgroup-1.0.xsd) unless @ns;
- $msg->command([$command,'nsgroup:'.$command,sprintf('xmlns:nsgroup="%s" xsi:schemaLocation="%s %s"',@ns)]);
-
+ $msg->command([$command,'nsgroup:'.$command,sprintf('xmlns:nsgroup="%s" xsi:schemaLocation="%s %s"',$msg->nsattrs('nsgroup'))]);
  return map { ['nsgroup:name',$_] } @gn;
 }
 
 sub add_nsname
 {
  my ($ns)=@_;
- return () unless (defined($ns));
+ return () unless defined $ns;
  my @a;
- if (! ref($ns))
+ if (! ref $ns)
  {
   @a=($ns);
- } elsif (ref($ns) eq 'ARRAY')
+ } elsif (ref $ns eq 'ARRAY')
  {
   @a=@$ns;
  } elsif (Net::DRI::Util::isa_nsgroup($ns))
@@ -129,7 +125,7 @@ sub add_nsname
   @a=$ns->get_names();
  }
 
- foreach my $n (@a) 
+ foreach my $n (@a)
  {
   next if Net::DRI::Util::is_hostname($n);
   Net::DRI::Exception->die(1,'protocol/EPP',10,'Invalid host name: '.$n);
@@ -143,8 +139,7 @@ sub add_nsname
 
 sub check
 {
- my $epp=shift;
- my @hosts=@_;
+ my ($epp,@hosts)=@_;
  my $mes=$epp->message();
  my @d=build_command($epp,$mes,'check',\@hosts);
  $mes->command_body(\@d);
@@ -156,24 +151,23 @@ sub check_parse
  my $mes=$po->message();
  return unless $mes->is_success();
 
- my $ns=ns($mes);
+ my $ns=$mes->ns('nsgroup');
  my $chkdata=$mes->get_response($ns,'chkData');
- return unless $chkdata;
+ return unless defined $chkdata;
+
  foreach my $cd ($chkdata->getChildrenByTagNameNS($ns,'cd'))
  {
-  my $c=$cd->getFirstChild();
   my $nsgroup;
-  while($c)
+  foreach my $el (Net::DRI::Util::xml_list_children($cd))
   {
-   next unless ($c->nodeType() == 1); ## only for element nodes
-   my $n=$c->localname() || $c->nodeName();
+   my ($n,$c)=@$el;
    if ($n eq 'name')
    {
-    $nsgroup=$c->getFirstChild()->getData();
+    $nsgroup=$c->textContent();
     $rinfo->{nsgroup}->{$nsgroup}->{exist}=1-Net::DRI::Util::xml_parse_boolean($c->getAttribute('avail'));
     $rinfo->{nsgroup}->{$nsgroup}->{action}='check';
    }
-  } continue { $c=$c->getNextSibling(); }
+  }
  }
 }
 
@@ -191,28 +185,24 @@ sub info_parse
  my $mes=$po->message();
  return unless $mes->is_success();
 
- my $infdata=$mes->get_response(ns($mes),'infData');
- return unless $infdata;
+ my $infdata=$mes->get_response($mes->ns('nsgroup'),'infData');
+ return unless defined $infdata;
 
- my $ns=Net::DRI::Data::Hosts->new();
-
- my $c=$infdata->getFirstChild();
- while ($c)
+ my $ns=$po->create_local_object('hosts');
+ foreach my $el (Net::DRI::Util::xml_list_children($infdata))
  {
-  next unless ($c->nodeType() == 1); ## only for element nodes
-  my $name=$c->localname() || $c->nodeName();
-  next unless $name;
+  my ($name,$c)=@$el;
   if ($name eq 'name')
   {
-   $oname=$c->getFirstChild()->getData();
+   $oname=$c->textContent();
    $ns->name($oname);
    $rinfo->{nsgroup}->{$oname}->{exist}=1;
    $rinfo->{nsgroup}->{$oname}->{action}='info';
   } elsif ($name eq 'ns')
   {
-   $ns->add($c->getFirstChild()->getData());
+   $ns->add($c->textContent());
   }
- } continue { $c=$c->getNextSibling(); }
+ }
 
  $rinfo->{nsgroup}->{$oname}->{self}=$ns;
 }
