@@ -1,6 +1,6 @@
 ## Domain Registry Interface, IRIS LWZ Connection handling
 ##
-## Copyright (c) 2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2008-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,6 +18,7 @@
 package Net::DRI::Protocol::IRIS::LWZ;
 
 use strict;
+use warnings;
 
 use Net::DRI::Util;
 use Net::DRI::Exception;
@@ -26,11 +27,10 @@ use Net::DRI::Protocol::ResultStatus;
 
 use Net::DNS ();
 
-## The LWZ RFC says DEFLATE=RFC1951 ; the DENIC IRIS DCHK server uses RFC1950 for now but is planning to fix that, so for now we try to handle both cases nicely
-use IO::Uncompress::Inflate (); ## RFC1950
-use IO::Uncompress::RawInflate (); ## RFC1951
+use IO::Uncompress::RawInflate (); ## RFC1951 per the LWZ RFC
+use IO::Compress::RawDeflate ();
 
-our $VERSION=do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.7 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -60,7 +60,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2008-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -81,10 +81,8 @@ sub read_data # 3.1.2
  my $data;
  $sock->recv($data,4000) or die(Net::DRI::Protocol::ResultStatus->new_error('COMMAND_FAILED_CLOSING','Unable to read registry reply: '.$!,'en'));
  my $hdr=substr($data,0,1);
-
  die(Net::DRI::Protocol::ResultStatus->new_error('COMMAND_FAILED_CLOSING','Unable to read 1 byte header','en')) unless $hdr;
  # 3.1.3
- ##my @bits=split(//,unpack('B8',$hdr));
  $hdr=unpack('C',$hdr);
  my $ver=($hdr & (128+64)) >> 6;
  die(Net::DRI::Protocol::ResultStatus->new_error('COMMAND_SYNTAX_ERROR','Version unknown in header: '.$ver,'en')) unless $ver==0;
@@ -96,17 +94,11 @@ sub read_data # 3.1.2
 
  my $tid=substr($data,1,2);
  $tid=unpack('n',$tid);
-
  my $load=substr($data,3);
  if ($deflate)
  {
   my $load2;
   IO::Uncompress::RawInflate::rawinflate(\$load,\$load2) or die(Net::DRI::Protocol::ResultStatus->new_error('COMMAND_FAILED','Unable to uncompress payload: '.$IO::Uncompress::RawInflate::RawInflateError,'en'));
-  if ($load2!~/<response>/) ## Try temporarily fallback to RFC1950
-  {
-   $load2=undef;
-   IO::Uncompress::Inflate::inflate(\$load,\$load2) or die(Net::DRI::Protocol::ResultStatus->new_error('COMMAND_FAILED','Unable to uncompress payload: '.$IO::Uncompress::Inflate::InflateError,'en'));
-  }
   $load=$load2;
  }
 
@@ -121,15 +113,15 @@ sub write_message
  my $m=Net::DRI::Util::encode_utf8($msg);
  my $hdr='00001000'; ## 3.1.3 : V=0 RR=Request PD=no DS=yes Reserved PT=xml
 
- ## TODO : handle message payload deflation, as needed (the RFC says when over 1500 bytes
- ## However, pay attention to the fact that some server do not accept such messages, see 3.1.7 "no-inflation-support-error", this is the case of DENIC server !
- ## So either code that information per DRD, or try anyway & fallback based on reply (this will need multiple exchanges, so probably some changes in Net::DRI::Registry::process)
-
-# use IO::Compress::RawDeflate;
-# my $mm;
-# IO::Compress::RawDeflate::rawdeflate(\$m,\$mm);
-# $m=$mm;
-# $hdr='00011000';
+ ## If not specificed in DRD, other option is to try anyway & fallback based on reply (this will need multiple exchanges, so probably some changes in Net::DRI::Registry::process)
+ my $deflate=$msg->options()->{request_deflate};
+ if ($deflate==2 || ($deflate==1 && length $m > 1500)) ## Deflate if forced or if message is over 1500 bytes (per RFC)
+ {
+  my $mm;
+  IO::Compress::RawDeflate::rawdeflate( \$m,\$mm);
+  $m=$mm;
+  $hdr='00011000';
+ }
 
  my ($tid)=($msg->tid()=~m/(\d{6})$/); ## 16 digits, we need to convert to a 16-bit value, we take the microsecond part modulo 65535 (since 0xFFFF is reserved)
  $tid%=65535;
@@ -165,5 +157,13 @@ sub find_remote_server
  return ($r[0]->target(),$r[0]->port());
 }
 
+sub transport_default
+{
+ my ($self,$tname)=@_;
+ ## RFC4993 Section 4 gives recommandation for timeouts and retry algorithm
+ ## retry=5 is computed so that the whole sequence stops after 60 seconds: t,p+2t,3/2*(p+2)-2+4t,3/2*3/2*(p+2)-2+8t, ...
+ return (defer => 1, close_after => 1, socktype=>'udp', timeout => 1, pause => 2, retry => 5);
+}
+
 ####################################################################################################
 1;
