@@ -1,6 +1,6 @@
 ## Domain Registry Interface, OpenSRS XCP Message
 ##
-## Copyright (c) 2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2008-2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,6 +18,7 @@
 package Net::DRI::Protocol::OpenSRS::XCP::Message;
 
 use strict;
+use warnings;
 
 use XML::LibXML ();
 
@@ -28,7 +29,7 @@ use Net::DRI::Util;
 use base qw(Class::Accessor::Chained::Fast Net::DRI::Protocol::Message);
 __PACKAGE__->mk_accessors(qw(version client_auth command command_attributes response_attributes response_code response_text response_is_success));
 
-our $VERSION=do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -58,7 +59,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2008-2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -118,11 +119,9 @@ our %CODES=( 200 => 1000,
 sub result_status
 {
  my $self=shift;
- return Net::DRI::Protocol::ResultStatus->new_success('COMMAND_SUCCESSFUL',$self->response_text()) if ($self->response_is_success());
-
+ return Net::DRI::Protocol::ResultStatus->new_success($self->response_text()) if $self->response_is_success();
  my $code=$self->response_code();
- my $eppcode=(!defined($code) || !exists($CODES{$code}))? 'GENERIC_ERROR' : $CODES{$code};
-
+ my $eppcode=(defined $code && exists $CODES{$code})? $CODES{$code} : 'COMMAND_FAILED';
  return Net::DRI::Protocol::ResultStatus->new('opensrs_xcp',$code,$eppcode,$self->response_is_success(),$self->response_text(),'en');
 }
 
@@ -161,7 +160,7 @@ sub _obj2dt
 {
  my ($in)=@_;
  my @r;
- foreach my $el ((ref($in) eq 'ARRAY')? @$in : ($in))
+ foreach my $el ($in)
  {
   my $ref=ref($el);
   if (!$ref)
@@ -174,7 +173,11 @@ sub _obj2dt
    {
     $k=~s/"/&quot;/g;
     my $v=$el->{$k};
-    push @c,sprintf('<item key="%s">%s</item>',$k,ref($v)? _obj2dt($v) : Net::DRI::Util::xml_escape($v));
+    if (!defined($v)) {
+     push @c,sprintf('<item key="%s" />',$k);
+    } else {
+     push @c,sprintf('<item key="%s">%s</item>',$k,ref($v)? _obj2dt($v) : Net::DRI::Util::xml_escape($v));
+    }
    }
    push @r,sprintf('<dt_assoc>%s</dt_assoc>',join('',@c));
   } elsif ($ref eq 'ARRAY')
@@ -190,7 +193,7 @@ sub _obj2dt
    push @r,sprintf('<dt_scalarref>%s</dt_scalarref>',Net::DRI::Util::xml_escape($$el)); ## defined in specifications, but not really used ?
   } else
   {
-   Net::DRI::Exception::err_assert('_obj2dt can not deal with data '.$el);
+   Net::DRI::Exception::err_assert('_obj2dt cannot deal with data '.$el);
   }
  }
  return @r;
@@ -202,11 +205,11 @@ sub _dt2obj
  my $c=$doc->getFirstChild();
  return unless defined($c);
  while (defined($c) && $c->nodeType()!=1) { $c=$c->getNextSibling(); }
- return $doc->getFirstChild()->getData() unless (defined($c) && $c->nodeType()==1);
+ return $doc->textContent() unless (defined($c) && $c->nodeType()==1);
  my $n=$c->nodeName();
  if ($n eq 'dt_scalar')
  {
-  return $c->getFirstChild()->getData();
+  return $c->textContent();
  } elsif ($n eq 'dt_assoc')
  {
   my %r;
@@ -236,14 +239,14 @@ sub parse
  my $parser=XML::LibXML->new();
  my $doc=$parser->parse_string($dr->as_string());
  my $root=$doc->getDocumentElement();
- Net::DRI::Exception->die(0,'protocol/OpenSRS/XCP',1,'Unsuccessfull parse, root element is not OPS_envelope') unless ($root->getName() eq 'OPS_envelope');
+ Net::DRI::Exception->die(0,'protocol/OpenSRS/XCP',1,'Unsuccessful parse, root element is not OPS_envelope but '.$root->getName()) unless ($root->getName() eq 'OPS_envelope');
 
  my $db=$root->getElementsByTagName('data_block');
- Net::DRI::Exception->die(0,'protocol/OpenSRS/XCP',1,'Unsuccessfull parse, did not found only one data_block node below root') unless ($db->size()==1);
- $db=$db->shift()->getChildrenByTagName('dt_assoc');
- Net::DRI::Exception->die(0,'protocol/OpenSRS/XCP',1,'Unsuccessfull parse, did not found one dt_assoc node directly below data_block') unless ($db->size()==1);
+ Net::DRI::Exception->die(0,'protocol/OpenSRS/XCP',1,'Unsuccessful parse, expected only one data_block node below root, found '.$db->size()) unless ($db->size()==1);
+ $db=$db->get_node(1)->getChildrenByTagName('dt_assoc');
+ Net::DRI::Exception->die(0,'protocol/OpenSRS/XCP',1,'Unsuccessful parse, expected one dt_assoc node directly below data_block, found '.$db->size()) unless ($db->size()==1);
 
- foreach my $item ($db->shift()->getChildrenByTagName('item'))
+ foreach my $item ($db->get_node(1)->getChildrenByTagName('item'))
  {
   my $key=$item->getAttribute('key');
   next if ($key eq 'protocol' || $key eq 'action' || $key eq 'object'); ## protocol is XCP, action is always REPLY, and we already have object in command()
@@ -254,17 +257,17 @@ sub parse
   }
   if ($key eq 'response_code') ## meaning is action-specific
   {
-   $self->response_code($item->getFirstChild()->getData());
+   $self->response_code($item->textContent());
    next;
   }
   if ($key eq 'response_text') ## meaning is action-specific
   {
-   $self->response_text($item->getFirstChild()->getData());
+   $self->response_text($item->textContent());
    next;
   }
   if ($key eq 'is_success') ## 0 if not successful, 1 if action was successful
   {
-   $self->response_is_success($item->getFirstChild()->getData());
+   $self->response_is_success($item->textContent());
    next;
   }
  }
