@@ -1,6 +1,7 @@
 ## Domain Registry Interface, RRI Protocol (DENIC-11)
 ##
-## Copyright (c) 2007,2008 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>. All rights reserved.
+## Copyright (c) 2007,2008,2009 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>. All rights reserved.
+##           (c) 2010 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -18,6 +19,7 @@
 package Net::DRI::Protocol::RRI;
 
 use strict;
+use warnings;
 
 use base qw(Net::DRI::Protocol);
 
@@ -26,7 +28,7 @@ use Net::DRI::Protocol::RRI::Message;
 use Net::DRI::Data::StatusList;
 use Net::DRI::Data::Contact::DENIC;
 
-our $VERSION=do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -56,7 +58,8 @@ Tonnerre Lombard, E<lt>tonnerre.lombard@sygroup.chE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2007,2008 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
+Copyright (c) 2007,2008,2009 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
+          (c) 2010 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -72,11 +75,10 @@ See the LICENSE file that comes with this distribution for more details.
 
 sub new
 {
- my ($c,$drd,$version,$extrah)=@_;
-
- my $self=$c->SUPER::new();
+ my ($c,$ctx,$rp)=@_;
+ my $self=$c->SUPER::new($ctx);
  $self->name('RRI');
- $version=Net::DRI::Util::check_equal($version,['2.0'],'2.0');
+ my $version=Net::DRI::Util::check_equal($rp->{version},['2.0'],'2.0');
  $self->version($version);
 
  foreach my $o (qw/ip status/) { $self->capabilities('host_update',$o,['set']); }
@@ -97,13 +99,13 @@ sub new
  $self->factories('message',sub { my $m=Net::DRI::Protocol::RRI::Message->new(@_); $m->ns($self->{ns}); $m->version($version); return $m; });
  $self->factories('status',sub { return Net::DRI::Data::StatusList->new(); });
  $self->factories('contact',sub { return Net::DRI::Data::Contact::DENIC->new(); });
- $self->_load($extrah);
+ $self->_load($rp);
  return $self;
 }
 
 sub _load
 {
- my ($self,$extrah)=@_;
+ my ($self,$rp)=@_;
 
  my @core=('Session','RegistryMessage','Domain','Contact');
  my @class=map { 'Net::DRI::Protocol::RRI::'.$_ } @core;
@@ -111,5 +113,11 @@ sub _load
  $self->SUPER::_load(@class);
 }
 
+sub transport_default
+{
+ my ($self)=@_;
+ return (protocol_connection => 'Net::DRI::Protocol::RRI::Connection', protocol_version => '2.0');
+}
+
 ####################################################################################################
 1;
