@@ -1,6 +1,6 @@
 ## Domain Registry Interface, Superclass of all Transport/* modules (hence virtual class, never used directly)
 ##
-## Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005-2011 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -25,7 +25,7 @@ __PACKAGE__->mk_accessors(qw/name version retry pause trace timeout defer curren
 
 use Net::DRI::Exception;
 
-our $VERSION=do { my @r=(q$Revision: 1.18 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.22 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -44,6 +44,18 @@ This is a superclass that should never be used directly, but only through its su
 During the new() call, subclasses will call this new() method, which expects a ref hash with some
 keys (other are handled by the subclasses), among which:
 
+=head2 defer
+
+do we open the connection right now (0) or later (1)
+
+=head2 timeout
+
+time to wait (in seconds) for server reply (default 60)
+
+=head2 retry
+
+number of times we try to send the message to the registry (default 2)
+
 =head2 trid
 
 (optional) code reference of a subroutine generating a transaction id when passed a name ; 
@@ -67,7 +79,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005-2011 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -88,31 +100,56 @@ sub new
 
  my $self={
  	   is_sync   => exists($ropts->{is_sync})? $ropts->{is_sync} : 1, ## do we need to wait for reply as soon as command sent ?
-           retry     => exists($ropts->{retry})?   $ropts->{retry}   : 1,  ## by default, we will try once only
+           retry     => exists($ropts->{retry})?   $ropts->{retry}   : 2,  ## by default, we will try once only
            pause     => exists($ropts->{pause})?   $ropts->{pause}   : 10, ## time in seconds to wait between two retries
-#           trace     => exists($ropts->{trace})?   $ropts->{trace}   : 0, ## NOT IMPL
-           timeout   => exists($ropts->{timeout})? $ropts->{timeout} : 0,
+           timeout   => exists($ropts->{timeout})? $ropts->{timeout} : 60,
            defer     => exists($ropts->{defer})?   $ropts->{defer}   : 0, ## defer opening connection as long as possible (irrelevant if stateless) ## XX maybe not here, too low
-           logging   => exists($ropts->{logging})? $ropts->{logging} : $ndr->logging(),
+           logging   => $ndr->logging(),
            trid_factory => (exists($ropts->{trid}) && (ref($ropts->{trid}) eq 'CODE'))? $ropts->{trid} : $ndr->trid_factory(),
            current_state => undef, ## for stateless transport, otherwise 0=close, 1=open
            has_state     => undef, ## do we need to open a session before sending commands ?
            transport     => undef, ## will be defined in subclasses
            time_creation => time(),
+           logging_ctx => { registry => $ndr->name(), profile => $pname, protocol => $ctx->{protocol}->name() },
           };
 
- if (exists($ropts->{log_fh}) && defined($ropts->{log_fh}))
- {
-  print STDERR 'log_fh is deprecated and will not be used now, please use new Logging framework',"\n";
- }
-
  bless $self,$class;
- $self->log_setup_channel($class,'transport',{ registry => $ndr, profile => $pname }); ## if we need the transport name here, we will have to put that further below, in another method called after new() ; otherwise we derive it from $class
+ $self->log_setup_channel($class,'transport',$self->{logging_ctx}); ## if we need the transport name here, we will have to put that further below, in another method called after new() ; otherwise we derive it from $class
  $self->log_output('debug','core',sprintf('Added transport %s for registry %s',$class,$ndr->name()));
  return $self;
 }
 
-sub transport_data { return shift->{transport}; }
+sub transport_data { my ($self,$data)=@_; return defined $data ? $self->{transport}->{$data} : $self->{transport}; }
+
+sub log_output
+{
+ my ($self,$level,$type,$data1,$data2)=@_;
+ return $self->logging()->output($level,$type,$data1) unless defined $data2;
+ $self->{logging_ctx}->{transport}=$self->name().'/'.$self->version() unless exists $self->{logging_ctx}->{transport};
+ return $self->logging()->output($level,$type,{ %{$self->{logging_ctx}}, %$data1, %$data2 });
+}
+
+## WARNING : this is a preliminary implementation of this new feature, it WILL change
+sub protocol_parse
+{
+ my ($to,$po,$otype,$oaction,$dr,$trid,$dur,$sent)=@_;
+ my ($rc,$rinfo)=$po->reaction($otype,$oaction,$dr);
+
+ $rinfo->{session}->{exchange}->{transport}=$to->name().'/'.$to->version();
+
+ foreach my $v1 (values(%$rinfo))
+ {
+  foreach my $v2 (values(%{$v1}))
+  {
+   delete($v2->{result_status}) if exists $v2->{result_status};
+  }
+ }
+
+ $rinfo->{session}->{exchange}={ %{$rinfo->{session}->{exchange}}, duration_seconds => $dur, raw_command => defined $sent ? $sent->as_string() : undef, raw_reply => $dr->as_string(), object_type => $otype, object_action => $oaction };
+ $rc->_set_data($rinfo);
+## die($rc) unless $rc->is_success(); ## was done just after reaction before ## TODO maybe not necessary ? Tweak Registry::add_profile + search for other die in Transport/
+ return $rc;
+}
 
 sub send
 {
@@ -121,10 +158,10 @@ sub send
  my $ok=0;
 
  ## Try to reconnect if needed
- $self->open_connection() if ($self->has_state() && !$self->current_state());
+ $self->open_connection($ctx) if ($self->has_state() && !$self->current_state()); ## TODO : grab result !
  ## Here $tosend is a Net::DRI::Protocol::Message object (in fact, a subclass of that), in perl internal encoding, no transport related data (such as EPP 4 bytes header)
- $self->log_output('notice','transport',{ctx=>$ctx,trid=>$ctx->{trid},phase=>'active',direction=>'out',driver=>$self->name().'/'.$self->version(),message=>$tosend});
- $ok=$self->$cb1($count,$tosend);
+ $self->log_output('notice','transport',$ctx,{phase=>'active',direction=>'out',message=>$tosend});
+ $ok=$self->$cb1($count,$tosend,$ctx);
  $self->time_used(time());
 
  Net::DRI::Exception->die(0,'transport',4,'Unable to send message to registry') unless $ok;
@@ -136,17 +173,32 @@ sub receive
  Net::DRI::Exception::err_insufficient_parameters() unless ($cb1 && (ref($cb1) eq 'CODE'));
 
  my $ans;
- $ans=$self->$cb1($count); ## a Net::DRI::Data::Raw object
- Net::DRI::Exception->die(0,'transport',5,'Unable to receive message from registry') unless defined($ans);
+ $ans=$self->$cb1($count,$ctx); ## a Net::DRI::Data::Raw object
+ Net::DRI::Exception->die(0,'transport',5,'Unable to receive message from registry') unless defined $ans;
  ## $ans should have been properly decoded into a native Perl string
- $self->log_output('notice','transport',{ctx=>$ctx,trid=>$ctx->{trid},phase=>'active',direction=>'in',driver=>$self->name().'/'.$self->version(),message=>$ans});
+ $self->log_output('notice','transport',$ctx,{phase=>'active',direction=>'in',message=>$ans});
  return $ans;
 }
 
 sub try_again ## TO BE SUBCLASSED
 {
- my ($self,$po,$err,$count,$istimeout,$step,$rpause,$rtimeout)=@_; ## $step is 0 before send, 1 after, and 2 after receive successful
- return 0; ## should return 1 if we try again, or 0 if we should stop processing now
+ my ($self,$ctx,$po,$err,$count,$istimeout,$step,$rpause,$rtimeout)=@_; ## $step is 0 before send, 1 after, and 2 after receive successful
+ ## Should return 1 if we try again, or 0 if we should stop processing now
+ return ($istimeout && ($count <= $self->{retry}))? 1 : 0;
+}
+
+sub open_connection
+{
+ my ($self,$ctx)=@_;
+ return unless $self->has_state();
+ Net::DRI::Exception::method_not_implemented('open_connection',$self);
+}
+
+sub end
+{
+ my ($self,$ctx)=@_;
+ return unless $self->has_state();
+ Net::DRI::Exception::method_not_implemented('end',$self);
 }
 
 ####################################################################################################
@@ -154,23 +206,9 @@ sub try_again ## TO BE SUBCLASSED
 ## Pass a true value if you want the connection to be automatically redone if the ping failed
 sub ping
 {
- my $self=shift;
+ my ($self,$autorecon)=@_;
  return unless $self->has_state();
- Net::DRI::Exception::err_method_not_implemented();
-}
-
-sub open_connection
-{
- my $self=shift;
- return unless $self->has_state();
- Net::DRI::Exception::err_method_not_implemented();
-}
-
-sub end
-{
- my $self=shift;
- return unless $self->has_state();
- Net::DRI::Exception::err_method_not_implemented();
+ Net::DRI::Exception::method_not_implemented('ping',$self);
 }
 
 ####################################################################################################
